# -*- coding: utf-8 -*-
# test_synthesizeNTF.py
# This module provides the tests for the synthesizeNTF function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the synthesizeNTF() function.
"""

import unittest
import numpy as np

from nose.tools import raises

from deltasigma import synthesizeNTF
from deltasigma._utils import cplxpair as cpx

class TestSynthesizeNTF(unittest.TestCase):
    """Test class for synthesizeNTF()"""

    def setUp(self):
        pass

    def test_synthesizeNTF_1(self):
        """Test function for synthesizeNTF() 1/15"""
        # synthesizeNTF should have as default values:
        # order=3, osr=64, opt=0, H_inf=1.5, f0=0.0
        z, p, k = synthesizeNTF()
        zref = [1., 1., 1.]
        pref = [.6694, .7654 + .2793j, .7654 - .2793j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue( np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue( np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_2(self):
        """Test function for synthesizeNTF() 2/15"""
        # Up next: bandpass test
        z, p, k = synthesizeNTF(order=4, osr=32, opt=0, H_inf=1.3, f0=.33)
        zref = [-0.4818 + 0.8763j, -0.4818 - 0.8763j, -0.4818 + 0.8763j,
                -0.4818 - 0.8763j]
        pref = [-0.5125 - 0.7018j, -0.5125 + 0.7018j, -0.3233 - 0.8240j, 
                -0.3233 + 0.8240j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_3(self):
        """Test function for synthesizeNTF() 3/15"""
        # repeat with zeros optimization
        z, p, k = synthesizeNTF(opt=1)
        zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
        pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_4(self):
        """Test function for synthesizeNTF() 4/15"""
        z, p, k = synthesizeNTF(order=4, osr=32, opt=1, H_inf=1.3, f0=.33)
        zref = [-0.4567 + 0.8896j, -0.4567 - 0.8896j, -0.5064 + 0.8623j,
                -0.5064 - 0.8623j]
        pref = [-0.5125 - 0.7014j, -0.5125 + 0.7014j, -0.3230 - 0.8239j,
                -0.3230 + 0.8239j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_5(self):
        """Test function for synthesizeNTF() 5/15"""
        z, p, k = synthesizeNTF(opt=2)
        zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
        pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_6(self):
        """Test function for synthesizeNTF() 6/15"""
        z, p, k = synthesizeNTF(order=4, osr=32, opt=2, H_inf=1.3, f0=.33)
        zref = [-0.4818 + 0.8763j, -0.4818 - 0.8763j, -0.4818 + 0.8763j,
                -0.4818 - 0.8763j]
        pref = [-0.5125 - 0.7018j, -0.5125 + 0.7018j, -0.3233 - 0.8240j,
                -0.3233 + 0.8240j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_7(self):
        """Test function for synthesizeNTF() 7/15"""
        # opt = 3
        z, p, k = synthesizeNTF(opt=3)
        zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
        pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_8(self):
        """Test function for synthesizeNTF() 8/15"""
        z, p, k = synthesizeNTF(order=4, osr=32, opt=3, H_inf=1.3, f0=.33)
        zref = [-0.4567 + 0.8896j, -0.4567 - 0.8896j, -0.5064 + 0.8623j,
                -0.5064 - 0.8623j]
        pref = [-0.5125 - 0.7014j, -0.5125 + 0.7014j, -0.3230 - 0.8239j,
                -0.3230 + 0.8239j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_9(self):
        """Test function for synthesizeNTF() 9/15"""
        # opt = 4
        z, p, k = synthesizeNTF(opt=4)
        zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
        pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_10(self):
        """Test function for synthesizeNTF() 10/15"""
        # Up next: odd order lowpass w zero at center band test
        z, p, k = synthesizeNTF(order=5, osr=32, opt=4, H_inf=1.3, f0=0.0)
        zref = [1.0000 + 0.0000j, 0.9986 + 0.0531j, 0.9986 - 0.0531j,
                0.9960 + 0.0892j, 0.9960 - 0.0892j]
        pref = [0.8718 - 0.0840j, 0.8718 + 0.0840j, 0.9390 - 0.1475j,
                0.9390 + 0.1475j, 0.8491 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF_11(self):
        """Test function for synthesizeNTF() 11/15"""
        # zeros passed explicitly
        opt = [1.0000 + 0.0000j, 0.9986 + 0.06j, 0.9986 - 0.06j,
                0.9960 + 0.0892j, 0.9960 - 0.0892j]
        z, p, k = synthesizeNTF(order=5, osr=32, opt=opt, H_inf=1.3, f0=0.0)
        zref = [1.0000 + 0.0000j, 0.9986 + 0.06j, 0.9986 - 0.06j,
                0.9960 + 0.0892j, 0.9960 - 0.0892j]
        pref = [0.8718 - 0.0840j, 0.8718 + 0.0840j, 0.9390 - 0.1475j,
                0.9390 + 0.1475j, 0.8491 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cpx(z), cpx(zref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(cpx(p), cpx(pref), atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    @raises(ValueError)
    def test_synthesizeNTF_12(self):
        """Test function for synthesizeNTF() 12/15"""
        # f0 > 0.5 -> ValueError
        z, p, k = synthesizeNTF(order=2, osr=32, opt=0, H_inf=1.3, f0=0.7)

    @raises(ValueError)
    def test_synthesizeNTF_13(self):
        """Test function for synthesizeNTF() 13/15"""
        # f0 > 0. and order odd -> ValueError
        z, p, k = synthesizeNTF(order=3, osr=32, opt=0, H_inf=1.3, f0=0.3)

    @raises(ValueError)
    def test_synthesizeNTF_14(self):
        """Test function for synthesizeNTF() 14/15"""
        # 1 < len(opt) < order
        z, p, k = synthesizeNTF(order=3, osr=32, opt=[0., 0.], H_inf=1.3, f0=0.3)

    @raises(ValueError)
    def test_synthesizeNTF_15(self):
        """Test function for synthesizeNTF() 15/15"""
        # order < len(opt)
        z, p, k = synthesizeNTF(order=3, osr=32, opt=[0., 0., 0., 0.], H_inf=1.3, f0=0.)




# -*- coding: utf-8 -*-
# _simulateSNR.py
# Module providing the simulateSNR function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the simulateSNR() function
"""

from __future__ import division

import collections
from warnings import warn

import numpy as np
from numpy.fft import fft, fftshift

from ._calculateSNR import calculateSNR
from ._mapQtoR import mapQtoR
from ._simulateDSM import simulateDSM
from ._utils import _get_zpk


def simulateSNR(arg1, osr, amp=None, f0=0, nlev=2, f=None, k=13,
                quadrature=False):
    """Determine the SNR for a delta-sigma modulator by using simulations.

    Simulate a delta-sigma modulator with sine wave inputs of various
    amplitudes and calculate the signal-to-noise ratio (SNR) in dB for each 
    input.

    Three alternative descriptions of the modulator can be used:

     * The modulator is described by a noise transfer function (NTF), provided
       as ``arg1`` and the number of quantizer levels (``nlev``).

     * Alternatively, the first argument to simulateSNR may be an ABCD matrix. 

     * Lastly, ``arg1`` may be a function taking the input signal as its
       sole argument.

    The band of interest is defined by the oversampling ratio (``osr``)
    and the center frequency (``f0``).

    The input signal is characterized by the ``amp`` vector and the ``f`` variable.
    A default value for ``amp`` is used if not supplied.

    ``f`` is the input frequency, normalized such that 1 -> fs;
    ``f`` is rounded to an FFT bin.

    Using sine waves located in FFT bins, the SNR is calculated as the ratio
    of the sine wave power to the power in all in-band bins other than those
    associated with the input tone. Due to spectral smearing, the input tone
    is not allowed to lie in bins 0 or 1. The length of the FFT is :math:`2^k`.

    If the NTF is complex, :func:`simulateQDSM` (which is slow, also available
    in a future release) is called.

    If ABCD is complex, :func:`simulateDSM` is used with the real equivalent
    of ABCD in order to speed up simulations.

    Future versions may accommodate STFs.

    **Parameters:**

    arg1 : scipy 'lti' object, or ndarray
        The first argument may be one of the various supported representations
        for a (SISO) transfer function or an ABCD matrix.

    osr : int
        The over-sampling ratio.

    amp : sequence, optional
        The amplitudes in dB, referred to the FS, for which the SNR is to be 
        evaluated. ``amp`` defaults to [-120 -110...-20 -15 -10 -9 -8 ... 0]dB,
        where 0 dB means a full-scale (peak value = nlev-1) sine wave.

    f0 : float, optional
        The center frequency. Normalized. Defaults to 0.

    nlev : int, optional
        Number of quantizer levels, defaults to 2.

    f : float, optional
        Test signal input frequency. Normalized. Rounded to an FFT bin.
        Defaults to:

    .. math::

        f = \\frac{1}{4\ \mathrm{OSR}}

    k : int, optional
        The number of samples used to compute the FFT is set by the integer `k`
        - default value 13 - through:

    .. math::

        N_{samples} = 2^k

    quadrature : boolean, optional
        Whether the delta sigma modulator is a quadrature modulator or not.
        Defaults to ``False``.

    .. note:: Setting ``quadrature`` to ``True`` results in a \\
        ``NotImplementedError`` being raised, as :func:`simulateQDSM` has not been \\
        implemented yet.

    **Returns:**

    snr : ndarray
        The SNR, from simulation.

    amp : ndarray
        The amplitudes corresponding to the SNR values.

    .. rubric:: Example:

    Compare the SNR vs input amplitude curve for a fifth-order modulator, as 
    determined by the describing function method (:func:`predictSNR`) with
    that determined by simulation (:func:`simulateSNR`).::

        import pylab as plt
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        snr_pred, amp, _, _, _ = predictSNR(H,OSR)
        snr, amp = simulateSNR(H, OSR)
        plt.plot(amp, snr_pred, 'b', label='Predicted')
        plt.hold(True)
        plt.plot(amp, snr, 'go', label='Simulated')
        plt.grid(True)
        figureMagic([-100, 0], 10, None,
                    [0, 100], 10, None)
        plt.xlabel('Input Level, dB')
        plt.ylabel('SNR, dB')
        s = 'peak SNR = %4.1fdB\\n' % max(snr)
        plt.text(-65, 15, s, horizontalalignment='left')
        plt.legend(loc='best')

    .. plot::

        import pylab as plt
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        snr_pred, amp, _, _, _ = predictSNR(H,OSR)
        snr, amp = simulateSNR(H, OSR)
        plt.plot(amp, snr_pred, 'b', label='Predicted')
        plt.hold(True)
        plt.plot(amp, snr, 'go', label='Simulated')
        plt.grid(True)
        figureMagic([-100, 0], 10, None,
                    [0, 100], 10, None)
        plt.xlabel('Input Level, dB')
        plt.ylabel('SNR, dB')
        s = 'peak SNR = %4.1fdB\\n' % max(snr)
        plt.text(-65, 15, s, horizontalalignment='left')
        plt.legend(loc='best')

    """
    # Look at arg1 and decide if the system is quadrature
    quadrature_ntf = False
    if callable(arg1):
        pass
    elif hasattr(arg1, '__class__') and arg1.__class__.__name__ == 'lti':
        # scipy LTI object
        for roots in _get_zpk(arg1)[:2]:
            if np.any(np.abs(np.imag(np.poly(roots))) > 0.0001):
                quadrature = True
                quadrature_ntf = True
    else: # ABCD matrix
        if not np.all(np.imag(arg1) == 0):
            quadrature = True

    if amp is None:
        amp = np.concatenate((
                              np.arange(- 120, -20 + 1, 10),
                              np.array((-15,)),
                              np.arange(-10, 1)
                            ))
    elif not isinstance(amp, collections.Iterable):
        amp = np.array((amp, ))
    else:
        amp = np.asarray(amp)
    osr_mult = 2
    if f0 != 0 and not quadrature:
        osr_mult = 2*osr_mult
    if f is None or np.isnan(f):
        f = f0 + 0.5/(osr*osr_mult) # Halfway across the band
    M = nlev - 1
    if quadrature and not quadrature_ntf:
        # Modify arg1 (ABCD) and nlev so that simulateDSM can be used
        nlev = np.array([nlev, nlev]).reshape(1, -1)
        arg1 = mapQtoR(arg1)
    if abs(f - f0) > 1/(osr*osr_mult):
        warn('The input tone is out-of-band.')
    N = 2**k
    if N < 8*2*osr:
        warn('Increasing k to accommodate a large oversampling ratio.')
        k = np.array(np.ceil(np.log2(8*2*osr)), dtype=np.int64)
        N = 2**k
    F = np.round(f*N)
    if np.abs(F) <= 1:
        warn('Increasing k to accommodate a low input frequency.')
        # Want f*N >= 1
        k = np.ceil(np.log2(1./f))
        N = 2**k
        F = 2
    Ntransient = 100
    soft_start = 0.5*(1 - np.cos(2*np.pi/Ntransient * \
                                 np.arange(0, Ntransient/2)))
    if not quadrature:
        tone = M*np.sin(2*np.pi*F/N*np.arange(0, N + Ntransient))
        tone[:Ntransient/2] = tone[:Ntransient/2] * soft_start
    else:
        tone = M * np.exp(2j*np.pi*F/N * np.arange(0, N + Ntransient))
        tone[:Ntransient/2] = tone[:Ntransient/2] * soft_start
        if not quadrature_ntf:
            tone = np.hstack((np.real(tone), np.imag(tone)))
    # create a Hann window
    window = 0.5*(1 - np.cos(2*np.pi*np.arange(0, N)/N))
    if quadrature:
        window = np.vstack((window, window))
    if f0 == 0:
        inBandBins = int(N/2) + np.arange(3,
                                     np.round(N/osr_mult/osr) + 1,
                                     dtype=np.int32)
        F = F - 2
    else:
        f1 = np.round(N*(f0 - 1./osr_mult/osr))
        # Should exclude DC
        inBandBins = int(N/2) + np.arange(f1,
                                     np.round(N*(f0 + 1./osr_mult/osr)) + 1,
                                     dtype=np.int32)
        F = F - f1 + 1
    snr = np.zeros(amp.shape)
    i = 0
    for A in np.power(10.0, amp/20):
        if callable(arg1):
            v = arg1(A*tone)
        else:
            if quadrature_ntf:
                raise NotImplementedError("simulateQDSM has not been \
                                           implemented yet.")
                #v = simulateQDSM(A*tone, arg1, nlev)
            else:
                v, _, _, _ = simulateDSM(A*tone, arg1, nlev)
                if quadrature:
                    v = v[0, :] + 1j*v[1, :]
        hwfft = fftshift(fft(window*v[Ntransient:N + Ntransient]))
        snr[i] = calculateSNR(hwfft[inBandBins - 1], F)
        i += 1
    return snr, amp



# -*- coding: utf-8 -*-
# test_plotPZ.py
# This module provides the tests for the plotPZ function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the plotPZ() function.
"""

import unittest
import numpy as np
import pylab as plt

from deltasigma import plotPZ

class TestPlotPZ(unittest.TestCase):
    """Test class for plotPZ()"""

    def setUp(self):
        pass

    def test_plotPZ_1(self):
        """Test function for plotPZ() 1/2"""
        plt.figure()
        self.assertIsNone(plotPZ(((1, .2), (1, 0, .10)), color=('r', 'b'),
                                 showlist=True))

    def test_plotPZ_2(self):
        """Test function for plotPZ() 2/2"""
        plt.figure()
        self.assertIsNone(plotPZ(((1, 0, .10), (1, .2, .01)), showlist=False))




# -*- coding: utf-8 -*-
# _peakSNR.py
# Module providing peakSNR
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the peakSNR function.
"""

from __future__ import division

import numpy as np
from numpy.linalg import lstsq

from ._config import _debug
from ._dbv import dbv


def peakSNR(snr, amp):
    """Find the SNR peak by fitting the SNR curve.

    A smooth curve is fitted to the top end of the SNR vs input amplitude
    data with Legendre's least-squares method.

    The curve fitted to the data is:

    .. math::

        y = ax + \\frac{b}{x - c}

    **Parameters:**

    snr : 1D ndarray or array-like
          Signal to Noise Ratio array, expressed in decibels (dB).

    amp : 1D ndarray or array-like
          Amplitude array, expressed in decibels (dB).

    The two arrays ``snr`` and ``amp`` should have the same size.

    **Returns:**

    (peak_snr, peak_amp) : tuple
                           The peak SNR value and its corresponding amplitude,
                           expressed in dB.
    """

    # sanitize inputs
    snr = np.atleast_1d(np.asarray(snr))
    amp = np.atleast_1d(np.asarray(amp))
    # Delete garbage data
    for check in (np.isinf, np.isnan):
        i = check(snr)
        snr = np.delete(snr, np.where(i))
        amp = np.delete(amp, np.where(i))
    for x in (snr, amp):
        if len(x.shape) > 1 and np.prod(x.shape) != max(x.shape):
            raise ValueError("snr and amp should be vectors (ndim=1)" +
                             "snr.shape: %s, amp.shape: %s" % (snr.shape, amp.shape))
    # All good
    max_snr = np.max(snr)
    i = np.flatnonzero(snr > max_snr - 10)
    min_i = np.min(i)
    max_i = np.max(i)
    j = np.flatnonzero(snr[min_i:max_i + 1] < max_snr - 15)
    if j:
        max_i = min_i + np.min(j) - 2
        i = np.arange(min_i, max_i + 1)
    snr = 10.0**(snr[i]/20)
    amp = 10.0**(amp[i]/20)
    #n = max(i.shape) # unused variable, REP?
    c = np.max(amp)*1.05
    # fit y = ax + b/(x-c) to the data
    A = np.hstack((amp.reshape(-1, 1), 1.0/(amp.reshape(-1, 1) - c)))
    ab = np.linalg.lstsq(A, snr.reshape((-1, 1)))[0]
    peak_amp = c - np.sqrt(ab[1, 0]/ab[0, 0])
    peak_snr = np.dot(np.array([[peak_amp, 1./(peak_amp-c)]]), ab) #None check mcode
    peak_snr = dbv(peak_snr)
    peak_amp = dbv(peak_amp)
    if _debug:
        import pylab as plt
        pred = np.dot(A, ab)
        hold = plt.ishold()
        plt.hold(True)
        plt.plot(dbv(amp), dbv(pred), '-', color='b')
        plt.hold(hold)
    return peak_snr, peak_amp



# -*- coding: utf-8 -*-
# _evalTF.py
# This module provides the evalTF function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the evalTF() function, used to evaluate a tf at the 
point(s) given by the user.
"""

import numpy as np

from ._evalRPoly import evalRPoly
from ._utils import _get_zpk, _is_num_den, _is_zpk


def evalTF(tf, z):
    """Evaluates the rational function ``tf`` at the point(s)
    given in ``z``.

    **Parameters:**

    tf : object
        the LTI description of the CT system, which can be in one of the
        following forms:

    * an LTI object,
    * a zpk tuple,
    * a (num, den) tuple,
    * an ABCD matrix (internally converted to zpk representation),
    * a list-like containing the A, B, C, D matrices (also internally converted to zpk representation).

    z : scalar or 1d ndarray
        The z values for which ``tf`` is to be evaluated.

    **Returns:**

    tf(z) : scalar or 1d-ndarray
        The result.

    """
    # Original comment in deltasig:
    # In Matlab 5, the ss/freqresp() function does nearly the same thing.

    # in our case a transfer function is a scipy LTI object
    if (hasattr(tf, 'inputs') and not tf.inputs == 1) or \
       (hasattr(tf, 'outputs') and not tf.outputs == 1):
        raise TypeError("Only SISO transfer functions can be evaluated.")

    if hasattr(tf, 'num') and hasattr(tf, 'den'):
        h = np.polyval(tf.num, z) / np.polyval(tf.den, z)
    elif (hasattr(tf, 'zeros') and hasattr(tf, 'poles')):
        # LTI objects have poles and zeros
        zeros = tf.zeros
        poles = tf.poles
        k = tf.gain
        h = k*evalRPoly(zeros, z)/evalRPoly(poles, z)
    # we try not to convert the given representation to another one
    elif _is_num_den(tf):
        num, den = tf[0], tf[1]
        h = np.polyval(num, z)/np.polyval(den, z)
    elif _is_zpk(tf):
        zeros, poles, k = tf
        h = k*evalRPoly(zeros, z)/evalRPoly(poles, z)
    else:
        # ABCD and A, B, C, D will be converted through _utils
        zeros, poles, k = _get_zpk(tf)
        h = k*evalRPoly(zeros, z)/evalRPoly(poles, z)
    return h



# -*- coding: utf-8 -*-
# _realizeNTF.py
# Module providing realizeNTF
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing realizeNTF()
"""

from __future__ import division, print_function

from warnings import warn

import numpy as np

from ._calculateTF import calculateTF
from ._evalRPoly import evalRPoly
from ._evalTF import evalTF
from ._stuffABCD import stuffABCD
from ._utils import _get_zpk, carray, cplxpair


def realizeNTF(ntf, form='CRFB', stf=None):
    """Convert an NTF into coefficients for the desired structure.

    **Parameters:**

    ntf : a zpk transfer function
        The modulator noise transfer function (NTF)

    form : string
        A structure identifier.

    Supported structures are:

    * *"CRFB"*: Cascade of resonators, feedback form.

    * *"CRFF"*: Cascade of resonators, feedforward form.

    * *"CIFB"*: Cascade of integrators, feedback form.

    * *"CIFF"*: Cascade of integrators, feedforward form.

    * *"CRFBD"*: CRFB with delaying quantizer.

    * *"CRFFD"*: CRFF with delaying quantizer.

    * *"PFF"*: Parallel feed-forward.

    * *"Stratos"*: A CIFF-like structure with non-delaying resonator feedbacks,
      contributed to the MATLAB Delta Sigma toolbox in 2007 by Jeff Gealow.

    See the accompanying documentation (:ref:`topologies-diagrams`) for
    block diagrams of each structure.

    .. note:

        The order of the NTF zeros must be (real, complex conj. pairs).
        The order of the zeros is used when mapping the NTF onto the chosen
        topology.

    stf : a zpk transfer function, optional
        the Signal Transfer Function

    **Returns:**

    a, g, b, c : tuple of ndarrays
        the coefficients for the desired structure

    **Example:**

    Determine the coefficients for a 5th-order modulator with the
    cascade-of-resonators structure, feedback (CRFB) form.::

        from deltasigma import synthesizeNTF, realizeNTF
        H = synthesizeNTF(5, 32, 1)
        a, g, b, c = realizeNTF(H,'CRFB')

    Returns the values::

        a: 0.0007, 0.0084, 0.055, 0.2443, 0.5579
        g: 0.0028, 0.0079
        b: 0.0007, 0.0084, 0.055, 0.2443, 0.5579, 1.0
        c: 1.0, 1.0, 1.0, 1.0, 1.0

    """

    # The basic idea is to equate the loop filter at a set of
    # points in the z-plane to L1  =  1-1/ntf at those points.

    # Code common to all functions\
    if (hasattr(ntf, 'inputs') and not ntf.inputs == 1) or \
       (hasattr(ntf, 'outputs') and not ntf.outputs == 1):
        raise TypeError("Only SISO transfer functions can be evaluated.")

    ntf_z, ntf_p, _ = _get_zpk(ntf)
    ntf_z = carray(ntf_z)
    ntf_p = carray(ntf_p)
    order = max(ntf_p.shape)
    order2 = int(np.floor(order / 2))
    odd = (order - 2 * order2 == 1)

    a = np.zeros((1, order))
    g = np.zeros((1, order2))
    b = np.zeros((1, order + 1))
    c = np.ones((1, order))
    T = np.zeros((order, order * 2), dtype='complex')

    # instead of asuming enforce roots order
    ntf_z = cplxpair(ntf_z)[::-1]

    N = 200
    min_distance = 0.09
    zSet = np.zeros((N, ), dtype='complex')
    j = 0
    for i in range(1, N + 1):
        z = 1.1 * np.exp(2j * np.pi * i / N)
        if np.all(np.abs(ntf_z - z) > min_distance):
            zSet[j] = z
            j = j + 1
    zSet = zSet[:j]
    if form == 'CRFB':
        # Find g
        # Assume the roots are ordered, real first, then cx conj. pairs
        for i in range(0, order2):
            g[0, i] = 2 * (1 - np.real(ntf_z[2 * i + 1 * odd]))
        L1 = np.zeros((1, order * 2), dtype='complex')
        # Form the linear matrix equation a*T* = L1
        for i in range(0, 2 * order):
            z = zSet[i]
            L1[0, i] = 1. - evalRPoly(ntf_p, z) / evalRPoly(ntf_z, z)
            Dfactor = (z - 1) / z
            product = 1
            for j in range(order, 0 + 1 * odd, -2):
                product = z / evalRPoly(ntf_z[j - 2:j], z) * product
                T[j - 1, i] = product * Dfactor
                T[j - 2, i] = product
            if odd:
                T[0, i] = product / (z - 1)
        a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
        if stf is None:
            b[0,:order] = a[0, :]
            b[0, order] = 1
    elif form == 'CRFF':
        # Find g
        # Assume the roots are ordered, real first, then cx conj. pairs
        for i in range(0, order2):
            g[0, i] = 2 * (1 - np.real(ntf_z[2 * i + 1 * odd]))
        L1 = np.zeros((1, order * 2), dtype='complex')
        # Form the linear matrix equation a*T*=L1
        for i in range(0, 2 * order):
            z = zSet[i]
            # L1(z) = 1-1/H(z)
            L1[0, i] = 1. - evalRPoly(ntf_p, z) / evalRPoly(ntf_z, z)
            if odd:
                Dfactor = z - 1
                product = 1 / Dfactor
                T[0, i] = product
            else:
                Dfactor = (z - 1) / z
                product = 1.
            for j in range(0 + 1 * odd, order, 2):
                product = z / evalRPoly(ntf_z[j:j + 2], z) * product
                T[j, i] = product * Dfactor
                T[j + 1, i] = product
        a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
        if stf is None:
            b = np.hstack((np.atleast_2d(1),
                           np.zeros((1, order - 1)),
                           np.atleast_2d(1)
                         ))
    elif form == 'CIFB':
        if any(abs(np.real(ntf_z) - 1) > 0.001):
            warn("The ntf's zeros have had their real parts set to one.")
        ntf_z = 1 + 1j * np.imag(ntf_z)
        for i in range(order2):
            g[0, i] = np.imag(ntf_z[2 * i + 1 * odd]) ** 2
        L1 = np.zeros((1, order * 2), dtype='complex')
        for i in range(order * 2):
            z = zSet[i]
            L1[0, i] = 1. - evalRPoly(ntf_p, z) / evalRPoly(ntf_z, z)
            Dfactor = z - 1.
            product = 1.
            for j in range(order - 1, 1 * odd, -2):
                product = product / evalRPoly(ntf_z[j - 1:j + 1], z)
                T[j, i] = product * Dfactor
                T[j - 1, i] = product
            if odd:
                T[0, i] = product / (z - 1.)
        a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
        if stf is None:
            b[0,:order] = a
            b[0, order] = 1.
    elif form == 'CIFF':
        if (abs(np.real(ntf_z) - 1) > 0.001).any():
            warn("The ntf's zeros have had their real parts set to one.")
        ntf_z = 1. + 1j * np.imag(ntf_z)
        for i in range(order2):
            g[0, i] = np.imag(ntf_z[2 * i + 1 * odd]) ** 2
        L1 = np.zeros((1, order * 2), dtype='complex')
        for i in range(order * 2):
            z = zSet[i]
            L1[0, i] = 1. - evalRPoly(ntf_p, z) / evalRPoly(ntf_z, z)
            Dfactor = (z - 1)
            if odd:
                product = 1. / (z - 1.)
                T[0, i] = product
            else:
                product = 1
            for j in range(0 + 1 * odd, order - 1, 2):
                product = product / evalRPoly(ntf_z[j:j + 2], z)
                T[j, i] = product * Dfactor
                T[j + 1, i] = product
        a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
        if stf is None:
            b = np.zeros((1, order + 1))
            b[0, 0] = 1.
            b[0, -1] = 1.
    elif form == 'CRFBD':
        for i in range(order2):
            g[0, i] = 2 * (1. - np.real(ntf_z[2 * i + 1 * odd]))
        L1 = np.zeros((1, order * 2), dtype='complex')
        for i in range(order * 2):
            z = zSet[i]
            L1[0, i] = 1. - evalRPoly(ntf_p, z) / evalRPoly(ntf_z, z)
            Dfactor = (z - 1.)
            product = 1. / z
            for j in range(order - 1, 0 + 1 * odd, -2):
                product = z / evalRPoly(ntf_z[j - 1:j + 1], z) * product
                T[j, i] = product * Dfactor
                T[j - 1, i] = product
            if odd:
                T[0, i] = product * z / (z - 1.)
        a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
        if stf is None:
            b = np.hstack((a, np.zeros((1, 1))))
            b[0, order] = 1.
    elif form == 'CRFFD':
        for i in range(order2):
            g[0, i] = 2 * (1. - np.real(ntf_z[2 * i + 1 * odd]))
        zL1 = np.zeros((1, order*2), dtype='complex')
        # Form the linear matrix equation a*T*=zL1
        for i in range(order * 2):
            z = zSet[i]
            # zL1 = z*(1-1/H(z))
            zL1[0, i] = z*(1. - 1.0/evalTF(ntf, z))
            if odd:
                Dfactor = (z - 1.) / z
                product = 1. / Dfactor
                T[0, i] = product
            else:
                Dfactor = z - 1
                product = 1
            for j in range(0 + 1 * odd, order, 2):
                product = z / evalRPoly(ntf_z[j:j + 2], z) * product
                T[j, i] = product * Dfactor
                T[j + 1, i] = product
        a = -np.real(np.linalg.lstsq(T.T, zL1.T)[0]).T
        if stf is None:
            b = np.hstack((np.atleast_2d(1),
                           np.zeros((1, order - 1)),
                           np.atleast_2d(1))
                         )
    elif form == 'PFF':
        warn("Untested code accessed: realizeNTF('PFF')")
        # Find g
        # Assume the roots are ordered, real first, then cx conj. pairs
        # with the secondary zeros after the primary zeros
        for i in range(order2):
            g[0, i] = 2 * (1. - np.real(ntf_z[2 * i + 1 * odd]))
        # Find the dividing line between the zeros
        theta0 = np.abs(np.angle(ntf_z[0]))
        # !! 0.5 radians is an arbitrary separation !!
        i = np.flatnonzero(np.abs(np.abs(np.angle(ntf_z)) - theta0) > 0.5)
        if i.shape[0]:
            order_1 = i[0] - 1
        else:
            order_1 = 0
        order_2 = order - order_1
        if i.shape[0] != order_2:
            raise ValueError('For the PFF form, the NTF zeros must ' +
                             'be sorted into primary and secondary zeros')
        odd_1 = order_1 % 2
        odd_2 = order_2 % 2
        L1 = np.zeros((1, order * 2), dtype='complex')
        # Form the linear matrix equation a*T*=L1
        for i in range(order * 2):
            z = zSet[i]
            # L1(z) = 1-1/H(z)
            L1[0, i] = 1. - evalRPoly(ntf_p, z) / evalRPoly(ntf_z, z)
            if odd_1:
                Dfactor = z - 1
                product = 1. / Dfactor
                T[0, i] = product
            else:
                Dfactor = (z - 1) / z
                product = 1
            for j in range(odd_1, order_1, 2):
                product = z / evalRPoly(ntf_z[j:j + 2], z) * product
                T[j, i] = product * Dfactor
                T[j + 1, i] = product
            if odd_2:
                Dfactor = z - 1
                product = 1. / Dfactor
                T[order_1, i] = product
            else:
                Dfactor = (z - 1) / z
                product = 1.
            for j in range(order_1 + odd_2, order, 2):
                product = z / evalRPoly(ntf_z[j:j + 2], z) * product
                T[j, i] = product * Dfactor
                T[j + 1, i] = product
        a = -np.real(np.linalg.lstsq(T.T, zL1.T)[0]).T
        if stf is None:
            b = np.hstack((np.atleast_2d(1),
                           np.zeros((1, order_1 - 1)),
                           np.atleast_2d(1),
                           np.zeros(1, order_2 - 1),
                           np.atleast_2d(1)
                         ))
    elif form == 'Stratos':
        # code copied from case 'CRFF':
        # Find g
        # Assume the roots are ordered, real first, then cx conj. pairs
        for i in range(order2):
            g[0, i] = 2 * (1 - np.real(ntf_z[2 * i + 1 * odd]))
        # code copied from case 'CIFF':
        L1 = np.zeros((1, order * 2), dtype='complex')
        # Form the linear matrix equation a*T*=L1
        for i in range(order * 2):
            z = zSet[i]
            # L1(z) = 1-1/H(z)
            L1[0, i] = 1. - evalRPoly(ntf_p, z) / evalRPoly(ntf_z, z)
            Dfactor = (z - 1.)
            if odd:
                product = 1. / (z - 1.)
                T[0, i] = product
            else:
                product = 1.
            for j in range(0 + 1 * odd, order - 1, 2):
                product = product / evalRPoly(ntf_z[j:j + 2], z)
                T[j, i] = product * Dfactor
                T[j + 1, i] = product
        a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
        if stf is None:
            b = np.hstack((
                           np.atleast_2d(1),
                           np.zeros((1, order - 1)),
                           np.atleast_2d(1)
                         ))
    if stf is not None:
        # Compute the TF from each feed-in to the output
        # and solve for coefficients which yield the best match
        # THIS CODE IS NOT OPTIMAL,  in terms of computational efficiency.
        stfList = []
        for i in range(0, order + 1):
            bi = np.zeros(1, order + 1)
            bi[i] = 1
            ABCD = stuffABCD(a, g, bi, c, form)
            if form[2:4] == 'FF':
                ABCD[0, order + 1] = -1
            _, stfListi = calculateTF(ABCD)
            stfList.append(stfListi)
        # Build the matrix equation b A  =  x and solve it.
        A = np.zeros((order + 1, max(zSet.shape)))
        for i in range(0, order + 1):
            A[i, :] = evalTF(stfList[i], zSet)
        x = evalTF(stf, zSet)
        x = x[:].T
        b = np.real(x / A)
    return a.squeeze(), g.squeeze(), b.squeeze(), c.squeeze()



# -*- coding: utf-8 -*-
# _evalRPoly.py
# This module provides the evalRPoly function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the evalRPoly() function, used to evaluate the value 
of a polynomial which is given in terms of its roots.
"""

import numpy as np

from ._utils import carray


def evalRPoly(roots, x, k=1):
	"""Compute the value of a polynomial which is given in terms of its roots.
	"""
	roots = carray(roots)
	y = k
	roots = roots[~np.isinf(roots)]        # remove roots at infinity
	for r in roots:
		y = y*(x - r)
	return y




# -*- coding: utf-8 -*-
# test_rms.py
# This module provides the tests for the rms function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the rms() function.
"""

import unittest
import numpy as np

from deltasigma import rms

class TestRms(unittest.TestCase):
    """Test class for rms()"""

    def setUp(self):
        self.tv = np.arange(100)
        self.res1 = np.sqrt(np.sum(self.tv**2.)/float(self.tv.shape[0]))
        self.res2 = np.sqrt((np.sum((self.tv - self.tv.mean())**2.)) \
                    /self.tv.shape[0])

    def test_rms_1(self):
        """Test function for rms() 1/2"""
        self.assertTrue(np.allclose(rms(self.tv), self.res1, rtol=1e-05,
                                    atol=1e-08))

    def test_rms_2(self):
        """Test function for rms() 2/2"""
        self.assertTrue(np.allclose(rms(self.tv, no_dc=True), self.res2,
                                    rtol=1e-05, atol=1e-08))




# -*- coding: utf-8 -*-
# _ds_quantize.py
# This module provides the ds_quantize function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the ds_quantize() function, used to quantize signals
according to a user-specified quantizer characteristic.
"""

import numpy as np


def ds_quantize(y, n=2):
    """Quantize ``y``

    Quantize a vector :math:`y` to:

     * an odd integer in :math:`[-n+1, n-1]`, if :math:`n` is even, or
     * an even integer in :math:`[-n+1, n-1]`, if :math:`n` is odd.

    The quantizer implementation details are repeated here from its
    documentation for the user's convenience:

       The quantizer is ideal, producing integer outputs centered
       about zero. Quantizers with an even number of levels are of
       the mid-rise type and produce outputs which are odd integers.
       Quantizers with an odd number of levels are of the mid-tread
       type and produce outputs which are even integers.

    This definition gives the same step height for both mid-rise
    and mid-tread quantizers.

    .. image:: ../doc/_static/quantizer_model.png
        :align: center
        :alt: Quantizer model


    **Parameters:**

    n : int or ndarray, optional
        The number of levels in the quantizer. If ``n`` is an integer,
        then all rows of y are fed to the same quantizer.
        If ``n`` is a column vector, each of its elements specifies
        how to quantize the rows of ``y``.

    **Returns:**

    v : ndarray
        The quantized vector.

    .. seealso::
        :func:`bquantize`, :func:`bunquantize`

    """
    assert (np.round(n, 0) == n).all()  # did we get an int or an array of int?
    if not isinstance(n, np.ndarray):
        n = n * np.ones(y.shape)  # we got an int
    else:
        assert len(n.shape) == 1 or 1 in n.shape
        n = (np.ones((max(n.shape), y.shape[1])).T * n).T

    i = (n % 2 == 0)
    v = np.zeros(y.shape)
    v[i] = 2 * np.floor(0.5 * y[i]) + 1     # mid-rise quantizer
    v[~i] = 2 * np.floor(0.5 * (y[~i] + 1)) # mid-tread quantizer

    # Limit the output
    L = n - 1
    for m in (-1, 1):
        i = (m * v > L)
        if i.any():
            v[i] = m * L[i]
    return v



# -*- coding: utf-8 -*-
# _lollipop.py
# The lollipop function: plot digital samples like it was a lollipop field.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Lollipop plotting function.
"""

from warnings import warn

import numpy as np
import pylab as plt


# Plot lollipops (o's and sticks)
# 
#       ^
#       | o     o     o         o o o   o o 
#       | |   o | o o | o     o | | | o | | 
#       | | o | | | | | | o o | | | | | | | 
#       +----------------------------------->

def lollipop(x, y, color=None, lw=2, ybot=0):
    """Plot lollipops (o's and sticks)
    
    **Parameters:**

    x, y : ndarrays
        The data to be plotted

    color : any matplotlib color, optional
            plotting color

    lw : float, optional
         line width value in points

    ybot : float, optional
           Dummy parameter available for compatiblity

    **Returns:**

    None
    
    **Example:**

    .. plot::

        import matplotlib.pyplot as plt
        import numpy as np
        from deltasigma import lollipop
        t = np.arange(1, 20)*1e-3
        f = 20.
        a = np.sin(2*np.pi*f*t)
        lollipop(t, a)
        plt.gcf().set_size_inches((8, 4))
        plt.grid(True)
        plt.show()
    """

    if ybot:
        warn('lollipop() got a non-zero ybot, but only ybot=0 is ' + \
             'supported. Setting ybot to 0.')
    markerline, stemlines, baseline = plt.stem(x, y, '-')
    if not color or color == 'None':
        color = stemlines[0].get_color()
    lolli_fmt = {'linewidth': lw, 'color': color}
    pop_fmt = {'mec': color, 'markerfacecolor':'None',  \
               'markersize':10, 'markeredgewidth': lw*1.1}
    plt.setp(markerline, **pop_fmt)
    plt.setp(stemlines, **lolli_fmt)
    plt.setp(baseline, 'color','k')



# -*- coding: utf-8 -*-
# _mapCtoD.py
# Module providing the mapCtoD function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the mapCtoD() function
"""

from __future__ import division, print_function

import collections
from warnings import warn

import numpy as np
from numpy.linalg import cond
from numpy.random import rand
from scipy.linalg import expm, inv, norm
from scipy.signal import cont2discrete, lti, ss2zpk

from ._constants import eps
from ._evalMixedTF import evalMixedTF
from ._padb import padb
from ._padr import padr
from ._utils import _getABCD


def mapCtoD(sys_c, t=(0, 1), f0=0.):
    """Map a MIMO continuous-time to an equiv. SIMO discrete-time system.

    The criterion for equivalence is that the sampled pulse response
    of the CT system must be identical to the impulse response of the DT system.
    i.e. If ``yc`` is the output of the CT system with an input ``vc`` taken
    from a set of DACs fed with a single DT input ``v``, then ``y``, the output
    of the equivalent DT system with input ``v`` satisfies:
    ``y(n) = yc(n-)`` for integer ``n``. The DACs are characterized by
    rectangular impulse responses with edge times specified in the t list.

    **Input:**

    sys_c : object
           the LTI description of the CT system, which can be:

     * the ABCD matrix,
     * a list-like containing the A, B, C, D matrices,
     * a list of zpk tuples (internally converted to SS representation).
     * a list of LTI objects

    t : array_like
        The edge times of the DAC pulse used to make CT waveforms
        from DT inputs. Each row corresponds to one of the system
        inputs; [-1 -1] denotes a CT input. The default is [0 1],
        for all inputs except the first.

    f0 : float
         The (normalized) frequency at which the Gp filters' gains are
         to be set to unity. Default 0 (DC).

    **Output:**

    sys : tuple
         the LTI description for the DT equivalent, in A, B, C, D
         representation.

    Gp : list of lists
         the mixed CT/DT prefilters which form the samples
         fed to each state for the CT inputs.

    **Example:**

    Map the standard second order CT modulator shown below to its CT
    equivalent and verify that its NTF is :math:`(1-z^{-1})^2`.

    .. image:: ../doc/_static/mapCtoD.png
        :align: center
        :alt: mapCtoD block diagram

    It can be done as follows::

        from __future__ import print_function
        import numpy as np
        from scipy.signal import lti
        from deltasigma import *
        LFc = lti([[0, 0], [1, 0]], [[1, -1], [0, -1.5]], [[0, 1]], [[0, 0]])
        tdac = [0, 1]
        LF, Gp = mapCtoD(LFc, tdac)
        LF = lti(*LF)
        ABCD = np.vstack((
                np.hstack((LF.A, LF.B)),
                np.hstack((LF.C, LF.D))
               ))
        NTF, STF = calculateTF(ABCD)
        print("NTF:") # after rounding to a 1e-6 resolution
        print("Zeros:", np.real_if_close(np.round(NTF.zeros, 6)))
        print("Poles:", np.real_if_close(np.round(NTF.poles, 6)))

    Prints::

        Zeros: [ 1.  1.]
        Poles: [ 0.  0.]

    Equivalent to::

               (z -1)^2
        NTF = ----------
                 z^2

    .. seealso:: R. Schreier and B. Zhang, "Delta-sigma modulators employing \
    continuous-time circuitry," IEEE Transactions on Circuits and Systems I, \
    vol. 43, no. 4, pp. 324-332, April 1996.
    """
    # You need to have A, B, C, D specification of the system
    Ac, Bc, Cc, Dc = _getABCD(sys_c)
    ni = Bc.shape[1]
    # Sanitize t
    if hasattr(t, 'tolist'):
        t = t.tolist()
    if (type(t) == tuple or type(t) == list) and np.isscalar(t[0]):
        t = [t] # we got a simple list, like the default value
    if not (type(t) == tuple or type(t) == list) and \
       not (type(t[0]) == tuple or type(t[0]) == list):
        raise ValueError("The t argument has an unrecognized shape")
    # back to business
    t = np.array(t)
    if t.shape == (1, 2) and ni > 1:
        t = np.vstack((np.array([[-1, -1]]), np.dot(np.ones((ni - 1, 1)), t)))
    if t.shape != (ni, 2):
        raise ValueError('The t argument has the wrong dimensions.')
    di = np.ones(ni).astype(bool)
    for i in range(ni):
        if t[i, 0] ==  -1 and t[i, 1] == -1:
            di[i] = False

    # c2d assumes t1=0, t2=1.
    # Also c2d often complains about poor scaling and can even produce
    # incorrect results.
    A, B, C, D, _ = cont2discrete((Ac, Bc, Cc, Dc), 1, method='zoh')
    Bc1 = Bc[:, ~di]

    # Examine the discrete-time inputs to see how big the
    # augmented matrices need to be.
    B1 = B[:, ~di]
    D1 = D[:, ~di]
    n = A.shape[0]
    t2 = np.ceil(t[di, 1]).astype(np.int_)
    esn = (t2 == t[di, 1]) and (D[0, di] != 0).T # extra states needed?
    npp = n + np.max(t2 - 1 + 1*esn)

    # Augment A to npp x npp, B to np x 1, C to 1 x np.
    Ap = padb(padr(A, npp), npp)
    for i in range(n + 1, npp):
        Ap[i, i - 1] = 1
    Bp = np.zeros((npp, 1))
    if npp > n:
        Bp[n, 0] = 1
    Cp = padr(C, npp)
    Dp = np.zeros((1, 1))

    # Add in the contributions from each DAC
    for i in np.flatnonzero(di):
        t1 = t[i, 0]
        t2 = t[i, 1]
        B2 = B[:, i]
        D2 = D[:, i]
        if t1 == 0 and t2 == 1 and D2 == 0: # No fancy stuff necessary
            Bp = Bp + padb(B2, npp)
        else:
            n1 = np.floor(t1)
            n2 = np.ceil(t2) - n1 - 1
            t1 = t1 - n1
            t2 = t2 - n2 - n1
            if t2 == 1 and D2 != 0:
                n2 = n2 + 1
                extraStateNeeded = 1
            else:
                extraStateNeeded = 0
            nt = n + n1 + n2
            if n2 > 0:
                if t2 == 1:
                    Ap[:n, nt - n2:nt] = Ap[:n, nt - n2:nt] + np.tile(B2, (1, n2))
                else:
                    Ap[:n, nt - n2:nt - 1] = Ap[:n, nt - n2:nt - 1] + np.tile(B2, (1, n2 - 1))
                    Ap[:n, (nt-1)] = Ap[:n, (nt-1)] + _B2formula(Ac, 0, t2, B2)
            if n2 > 0: # pulse extends to the next period
                Btmp = _B2formula(Ac, t1, 1, B2)
            else: # pulse ends in this period
                Btmp = _B2formula(Ac, t1, t2, B2)
            if n1 > 0:
                Ap[:n, n + n1 - 1] = Ap[:n, n + n1 - 1] + Btmp
            else:
                Bp = Bp + padb(Btmp, npp)
            if n2 > 0:
                Cp = Cp + padr(np.hstack((np.zeros((D2.shape[0], n + n1)), D2*np.ones((1, n2)))), npp)
    sys = (Ap, Bp, Cp, Dp)
    if np.any(~di):
        # Compute the prefilters and add in the CT feed-ins.
        # Gp = inv(sI - Ac)*(zI - A)/z*Bc1
        n, m = Bc1.shape
        Gp = np.empty_like(np.zeros((n, m)), dtype=object)
        # !!Make this like stf: an array of zpk objects
        ztf = np.empty_like(Bc1, dtype=object)
        # Compute the z-domain portions of the filters
        ABc1 = np.dot(A, Bc1)
        for h in range(m):
            for i in range(n):
                if Bc1[i, h] == 0:
                    ztf[i, h] = (np.array([]),
                                 np.array([0.]),
                                 -ABc1[i, h]) # dt=1
                else:
                    ztf[i, h] = (np.atleast_1d(ABc1[i, h]/Bc1[i, h]),
                                 np.array([0.]),
                                 Bc1[i, h]) # dt = 1
        # Compute the s-domain portions of each of the filters
        stf = np.empty_like(np.zeros((n, n)), dtype=object) # stf[out, in] = zpk
        for oi in range(n):
            for ii in range(n):
                # Doesn't do pole-zero cancellation
                stf[oi, ii] = ss2zpk(Ac, np.eye(n), np.eye(n)[oi, :],
                                     np.zeros((1, n)), input=ii)
                # scipy as of v 0.13 has no support for LTI MIMO systems
                # only 'MISO', therefore you can't write:
                # stf = ss2zpk(Ac, eye(n), eye(n), np.zeros(n, n)))
        for h in range(m):
            for i in range(n):
                # k = 1 unneded, see below
                for j in range(n):
                    # check the k values for a non-zero term
                    if stf[i, j][2] != 0 and ztf[j, h][2] != 0:
                        if Gp[i, h] is None:
                            Gp[i, h] = {}
                            Gp[i, h].update({'Hs':[list(stf[i, j])]})
                            Gp[i, h].update({'Hz':[list(ztf[j, h])]})
                        else:
                            Gp[i, h].update({'Hs':Gp[i, h]['Hs'] + [list(stf[i, j])]})
                            Gp[i, h].update({'Hz':Gp[i, h]['Hz'] + [list(ztf[j, h])]})
                        # the MATLAB-like cell code for the above statements would have
                        # been:
                        #Gp[i, h](k).Hs = stf[i, j]
                        #Gp[i, h](k).Hz = ztf[j, h]
                        #k = k + 1
        if f0 != 0: # Need to correct the gain terms calculated by c2d
            # B1 = gains of Gp @f0;
            for h in range(m):
                for i in range(n):
                    B1ih = np.real_if_close(evalMixedTF(Gp[i, h], f0))
                    # abs() used because ss() whines if B has complex entries...
                    # This is clearly incorrect.
                    # I've fudged the complex stuff by including a sign....
                    B1[i, h] = np.abs(B1ih) * np.sign(np.real(B1ih))
                    if np.abs(B1[i, h]) < 1e-09:
                        B1[i, h] = 1e-09 # This prevents NaN in "line 174" below
        # Adjust the gains of the pre-filters
        for h in range(m):
            for i in range(n):
                for j in range(max(len(Gp[i, h]['Hs']), len(Gp[i, h]['Hz']))):
                    # The next is "line 174"
                    Gp[i, h]['Hs'][j][2] = Gp[i, h]['Hs'][j][2]/B1[i, h]
        sys = (sys[0], # Ap
               np.hstack((padb(B1, npp), sys[1])), # new B
               sys[2], # Cp
               np.hstack((D1, sys[3]))) # new D
    return sys, Gp

def _B2formula(Ac, t1, t2, B2):
    if t1 == 0 and t2 == 0:
        term = B2
        return term
    n = Ac.shape[0]
    tmp = np.eye(n) - expm(-Ac)
    if cond(tmp) < 1000000.0:
        term = np.dot(((expm(-Ac*t1) - expm(-Ac*t2))*inv(tmp)), B2)
        return term
    # Numerical trouble. Perturb slightly and check the result
    ntry = 0
    k = np.sqrt(eps)
    Ac0 = Ac
    while ntry < 2:
        Ac = Ac0 + k*rand(n,n)
        tmp = np.eye(n) - expm(-Ac)
        if cond(tmp) < 1/np.sqrt(eps):
            ntry = ntry + 1
            if ntry == 1:
                term = np.dot(np.dot(expm(-Ac*t1) - expm(-Ac*t2), inv(tmp)), B2)
            else:
                term1 = np.dot(np.dot(expm(-Ac*t1) - expm(-Ac*t2), inv(tmp)), B2)
        k = k*np.sqrt(2)

    if norm(term1 - term) > 0.001:
        warn('Inaccurate calculation in mapCtoD.')
    return term



# -*- coding: utf-8 -*-
# test_sinc_decimate.py
# This module provides the tests for the sinc_decimate function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the sinc_decimate() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from nose.tools import raises

class TestSincDecimate(unittest.TestCase):
    """Test class for sinc_decimate()"""

    def test_sinc_decimate_1(self):
        """Test function for sinc_decimate() 1/4"""
        x = [1]*10
        self.assertTrue(np.allclose(ds.sinc_decimate(x, 1, 10), [1.]))

    def test_sinc_decimate_2(self):
        """Test function for sinc_decimate() 2/4"""
        x = [1]*10
        self.assertTrue(np.allclose(ds.sinc_decimate(x, 2, 5), [.6, 1.]))

    def test_sinc_decimate_3(self):
        """Test function for sinc_decimate() 3/4"""
        x = [1]*10
        x = np.cumsum(np.asarray(x))
        self.assertTrue(np.allclose(ds.sinc_decimate(x, 1, 5), [3., 8.]))
        self.assertTrue(np.allclose(ds.sinc_decimate(x, 2, 5), [1.4, 6.]))
        self.assertTrue(np.allclose(ds.sinc_decimate(x, 3, 3), [0.55555556, 3., 6.]))

    @raises(ValueError)
    def test_sinc_decimate_4(self):
        """Test function for sinc_decimate() 4/4"""
        x = [1]*10
        ds.sinc_decimate(x, 2, 15)




# -*- coding: utf-8 -*-
# _impL1.py
# This module provides the impL1 function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the impL1() function, computing the impulse response 
from the comparator output to the comparator input for a given NTF.
"""

import numpy as np
from scipy.signal import convolve, dimpulse

from ._padr import padr
from ._utils import _get_num_den, _is_num_den, _is_zpk


def impL1(arg1, n=10):
    """Impulse response evaluation for NTFs.

    Compute the impulse response from the comparator
    output to the comparator input for the given NTF.

    **Parameters:**

    arg1 : object
        The NTF, which may be represented as:

        * ZPK tuple,
        * num, den tuple,
        * A, B, C, D tuple,
        * ABCD matrix,
        * a scipy LTI object,
        * a sequence of the tuples of any of the above types (experimental).

    n : int
        is the (optional) number of time steps (default: 10), resulting in
        an impulse response with n+1 (default: 11) samples.

    This function is useful when verifying the realization
    of a NTF with a specified topology.

    **Returns:**

    y : ndarray
        The NTF impulse response

    .. note::

        In the original implementation of impL1 in delsig, there is a
        bug: impL1 calls MATLAB's impulse with tfinal=n, which means that
        the function will return the impulse response evaluated on the times
        [0, 1, 2 ... n], ie n+1 points. We keep the same behavior here,
        but we state clearly that n is the number of time steps.

    """
    if _is_num_den(arg1):
        num, den = arg1
        p = np.roots(den)
    elif _is_zpk(arg1):
        z, p, _ = arg1
        num = np.poly(z)
        den = np.poly(p)
    else:
        num, den = _get_num_den(arg1)
        p = np.roots(den)

    num = np.asarray(num)
    den = np.asarray(den)
    p = np.asarray(p)

    lf_den = padr(num, len(p)+1)
    lf_num = lf_den - den
    ts = np.arange(n + 1) # be coherent with the original toolbox
    all_lf = np.concatenate((lf_num, lf_den), axis=1)
    lf_num, lf_den = lf_num.squeeze(), lf_den.squeeze()
    if not np.allclose(np.imag(all_lf), np.zeros(all_lf.shape), atol=1e-9):
        # Complex loop filter
        lfr_den = np.real(convolve(lf_den, np.conj(lf_den))).squeeze()
        lfr_num = convolve(lf_num, np.conj(lf_den)).squeeze()
        lf_i = (np.real(lfr_num).tolist(), lfr_den.tolist(), 1)
        lf_q = (np.imag(lfr_num).tolist(), lfr_den.tolist(), 1)
        y = dimpulse(lf_i, t=ts)[1][0] + 1j*dimpulse(lf_q, t=ts)[1][0]
        y = y.squeeze()
    else:
        _, y = dimpulse((lf_num, lf_den, 1), t=ts)
        y = y[0].squeeze()
    return y



# -*- coding: utf-8 -*-
# _mapABCD.py
# Module providing the mapABCD function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the mapABCD() function
"""

from __future__ import division, print_function

import numpy as np

from ._utils import diagonal_indices


def mapABCD(ABCD, form='CRFB'):
    """Compute the coefficients for the specified structure.

    It is assumed that the ABCD matrix fits the topology.

    **Parameters:**

    ABCD : ndarray
        A state-space description of the modulator loop filter.
    form : str, optional
        See :func:`realizeNTF` for a list of supported structures.

    **Returns:**
    
    a : ndarray
        Feedback/feedforward coefficients from/to the quantizer. Length :math:`n`.
    g : ndarray
        Resonator coefficients. Length :math:`floor(n/2)`.
    b : ndarray
        Feed-in coefficients from the modulator input to each integrator. Length :math:`n + 1`.
    c : ndarray
        Integrator inter-stage coefficients. Length :math:`n`.

    .. seealso::

        * :func:`realizeNTF` for a list of supported structures.

        * :func:`stuffABCD`, the inverse function.

    """
    ABCD = np.copy(ABCD)
    order = ABCD.shape[0] - 1
    odd = order % 2
    even = 1 - odd
    diagonal = diagonal_indices(ABCD)
    subdiag = diagonal_indices(ABCD, -1)
    supdiag = [a[odd:order - 1:2] for a in diagonal_indices(ABCD, +1)]
    if form in ('CRFB', 'CIFB', 'CRFBD'):
        c = ABCD[subdiag]
        g = -ABCD[supdiag]
        if form == 'CRFB':
            dly = np.arange(1 + odd, order, 2)
            ABCD[dly, :] = ABCD[dly, :] \
                           - np.dot(np.diag(c[dly - 1]), ABCD[dly - 1, :])
        elif form == 'CRFBD':
            dly = np.arange(odd, order, 2)
            ABCD[dly, :] = ABCD[dly, :] + np.dot(np.diag(g), ABCD[dly + 1, :])
            if order > 2:
                coupl = np.arange(1 + even, order, 2)
                ABCD[coupl, :] = ABCD[coupl, :] \
                                 - np.dot(np.diag(c[coupl - 1]), ABCD[coupl - 1, :])
        a = -ABCD[:order, order + 1].T
        b = ABCD[:, order].T
    elif form == 'CRFF':
        a = np.zeros((order,))
        c = np.concatenate((
                            np.array((-ABCD[0, order + 1],)), 
                            ABCD[subdiag][:-1]
                          ))
        g = -ABCD[supdiag]
        if even:
            multg = np.arange(0, order, 2)
            ABCD[multg, :] = ABCD[multg, :] + np.dot(np.diag(g), ABCD[multg + 1, :])
        multc = np.arange(2, order, 2)
        ABCD[multc, :] = ABCD[multc, :] - np.dot(np.diag(c[multc]), ABCD[multc - 1, :])
        a[1:order:2] = ABCD[order, 1:order:2]
        for i in range(1, order, 2):
            ABCD[order, :] = ABCD[order, :] - a[i]*ABCD[i, :]
        a[:order:2] = ABCD[order, :order:2]
        b = ABCD[:, order].T
    elif form == 'CRFFD':
        order = order - 1
        odd = order % 2
        even = 1 - odd
        diagonal = diagonal_indices(ABCD[:order, :order])
        subdiag = diagonal_indices(ABCD[:order, :order], -1)
        supdiag = [a[odd:order:2] for a in 
                      diagonal_indices(ABCD[:order, :order], +1)]
        g = -ABCD[supdiag]
        c = np.concatenate((np.atleast_1d(-ABCD[0, order + 2]), ABCD[subdiag]))
        a = np.zeros((order, ))
        for i in range(0, order, 2):
            a[i] = ABCD[order, i]
            ABCD[order, :] = ABCD[order, :] - a[i]*ABCD[i, :]
        a[1:order:2] = ABCD[order, 1:order:2]
        b = ABCD[:order + 1, order + 1].T
        for i in range(1, order, 2):
            b[i] = b[i] - c[i]*b[i - 1]
            if odd:
                b[i] = b[i] + g[(i - 1)/2]*b[i + 1]
        yscale = ABCD[order + 1, order]
        a = a*yscale
        b[-1] = b[-1]*yscale
    elif form in ('CIFF', 'Stratos'):
        a = ABCD[order, :order]
        c = np.concatenate((np.atleast_1d(-ABCD[0, order + 1]),
                            ABCD[subdiag][:-1]))
        g = -ABCD[supdiag]
        b = ABCD[:, order].T
    else:
        raise ValueError('Form %s is not yet supported.' % form)

    a = np.atleast_1d(a.squeeze())
    g = np.atleast_1d(g.squeeze())
    b = np.atleast_1d(b.squeeze())
    c = np.atleast_1d(c.squeeze())
    return a, g, b, c



# -*- coding: utf-8 -*-
# _undbv.py
# This module provides the undbv function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the undbv() function.
"""
from __future__ import division

import numpy as np

from ._utils import carray, restore_input_form, save_input_form


def undbv(x):
    """Convert ``x`` from dB to voltage.

    The conversion is carried out according to the relationship:

    .. math::

        V_{\\mathrm{RMS}} = 10^{x/20}

    **Parameters:**

    x : scalar or sequence
        The signal in dB to be converted.

    **Returns:**

    Vrms : scalar or sequence
           The RMS voltage corresponding to x.

    .. seealso:: :func:`undbm`, :func:`undbp`, :func:`dbv`, :func:`db`

    """
    iform = save_input_form(x)
    x = carray(x)
    up = 10.**(x/20.)
    return restore_input_form(up, iform)



# -*- coding: utf-8 -*-
# test_pulse.py
# This module provides the tests for the pulse function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the pulse() function.
"""

import unittest
import pkg_resources
import numpy as np
import scipy.io

from nose.tools import raises

from deltasigma import pulse, axisLabels

class TestPulse(unittest.TestCase):
    """Test class for pulse()"""

    def setUp(self):
        self.H = ([1], [1, 2, 10])
        self.H0 = ([1], [1, 2, 10])
        self.H1 = ([2], [1, 2, 10])
        self.H2 = ([3], [1, 2, 10])
        fname = pkg_resources.resource_filename(__name__, "test_data/test_pulse.mat")
        self.pp2 = scipy.io.loadmat(fname)['pp']

    def test_pulse1(self):
        """Test function for pulse(): SISO 1/6"""
        # SISO
        pp = pulse([self.H], tp=(0., 1.), dt=.1, tfinal=10., nosum=False)
        pp = pp.reshape((pp.shape[0], 1))
        self.assertTrue(np.allclose(pp, self.pp2, atol=1e-6, rtol=1e-4))

    def test_pulse2(self):
        """Test function for pulse(): SIMO 2/6"""
        # SIMO
        # 1 input, 3 outputs
        pp = pulse([[self.H0, self.H1, self.H2]], tp=(0., 1.), dt=.1, tfinal=10., nosum=True)
        self.assertTrue(np.allclose(pp[:, 0, :], self.pp2, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(pp[:, 1, :], 2*self.pp2, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(pp[:, 2, :], 3*self.pp2, atol=1e-6, rtol=1e-4))

    def test_pulse3(self):
        """Test function for pulse(): MISO 3/6"""
        # MISO
        # 3 inputs, one output
        # we rely here on _pulse() zipping the input tfs list
        # [H0, H1, H2] becomes [[H0], [H1], [H2]]
        pp = pulse([self.H0, self.H1, self.H2], tp=[(0., 1.)]*3, dt=.1, tfinal=10., nosum=True)
        self.assertTrue(np.allclose(pp[:, :, 0], self.pp2, atol=1e-6, rtol=1e-3))
        self.assertTrue(np.allclose(pp[:, :, 1], 2*self.pp2, atol=1e-6, rtol=1e-3))
        self.assertTrue(np.allclose(pp[:, :, 2], 3*self.pp2, atol=1e-6, rtol=1e-3))

    # FIXME ALSO CHECK MIMO TFS

    @raises(ValueError)
    def test_pulse4_tp_scalar(self):
        """Test function for pulse(): fail if tp scalar 4/6"""
        H = ((1,), (0, 0))
        pulse(H, tp=1, dt=.1, tfinal=10)

    @raises(ValueError)
    def test_pulse5_tp_not_nx2(self):
        """Test function for pulse(): fail if tp.shape not (n, 2) 5/6"""
        H = ((1,), (0, 0))
        tp = ((1, 2, 3), (4, 5, 6))
        pulse(H, tp=1, dt=.1, tfinal=10)

    @raises(ValueError)
    def test_pulse6_S_tp_mismatch(self):
        """Test function for pulse(): fail if mismatched ndac, ni 6/6"""
        H = ((1,), (0, 0))
        S = [[H], [H], [H]]
        tp = ((1, 2), (3, 4), (5, 6))
        pulse(H, tp=1, dt=.1, tfinal=10)




# -*- coding: utf-8 -*-
# test_cancelPZ.py
# This module provides the tests for the cancelPZ() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds


class TestCancelPZ(unittest.TestCase):
    """Test function for calculateTF()"""
    def setUp(self):
        zt = np.array((1, 2, 3))
        pt = np.array((1 + 2e-5, 2 + .5e-5, 3))
        kt = 2
        zpkt = (zt, pt, kt)
        self.zr, self.pr, self.kr = ds.cancelPZ(zpkt, tol=1e-5)

    def test_zr(self):
        self.assertTrue(np.allclose(self.zr, (1.0, ), atol=1e-8, rtol=1e-6))

    def test_pr(self):
        self.assertTrue(
            np.allclose(self.pr, (1.0 + 2e-5, ), atol=1e-8, rtol=1e-6))

    def test_kr(self):
        self.assertTrue(np.allclose(self.kr, 2.0, atol=1e-8, rtol=1e-6))



# -*- coding: utf-8 -*-
# test_circ_smooth.py
# This module provides the tests for the circ_smooth() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds
import pkg_resources
from scipy.io import loadmat
from os.path import join


class TestCircSmooth(unittest.TestCase):
    """Test function for circ_smooth()"""

    def setUp(self):
        file_path = join('test_data', 'test_circ_smooth.mat')
        fname = pkg_resources.resource_filename(__name__, file_path)
        self.bt = loadmat(fname)['b']

    def test_circ_smooth(self):
        A = np.arange(1, 101)
        b = ds.circ_smooth(A, 16)
        self.assertTrue(np.allclose(self.bt, b, atol=1e-8, rtol=1e-5))



# -*- coding: utf-8 -*-
# test_evalF1.py
# This module provides the tests for the evalF1 function.
# Copyright 2014 Giuseppe Venturini and Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the evalF1() function.
"""

import unittest
import numpy as np

from deltasigma import evalF1

class TestEvalF1(unittest.TestCase):
    """Test class for evalF1()"""

    def setUp(self):
        pass

    def test_evalF1_1(self):
        """Test function for evalF1() 1/2"""
        r = evalF1([0.5, 1, 1.5, 2, 5, 10, 20, 30, 40.7, 50], 2, 23)
        ref = 0.544143311383570
        self.assertTrue(np.allclose((r,), (ref,)))

    def test_evalF1_2(self):
        """Test function for evalF1() 2/2"""
        r = evalF1([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 0.135)
        ref = 0.640058615996223
        self.assertTrue(np.allclose((r,), (ref,)))




# -*- coding: utf-8 -*-
# test_zinc.py
# This module provides the tests for the zinc function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the zinc() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class TestZinc(unittest.TestCase):
    """Test class for zinc()"""

    def setUp(self):
        self.ref = [1.0000, 0.9985, 0.9941, 0.9867, 0.9765, 0.9635,
                    0.9478, 0.9295, 0.9087, 0.8855, 0.8602, 0.8329,
                    0.8038, 0.7730, 0.7408, 0.7074, 0.6729, 0.6377,
                    0.6019, 0.5658, 0.5295, 0.4933, 0.4574, 0.4221,
                    0.3874, 0.3536, 0.3208, 0.2892, 0.2590, 0.2302,
                    0.2031, 0.1776, 0.1538, 0.1319, 0.1118, 0.0936,
                    0.0772, 0.0626, 0.0499, 0.0389, 0.0295, 0.0217,
                    0.0154, 0.0104, 0.0066, 0.0038, 0.0020, 0.0008,
                    0.0002, 0.0000, 0.0000]
        self.f = np.arange(0, 0.51, 0.01)

    def test_zinc(self):
        """Test function for zinc()"""
        test = ds.zinc(self.f, 2, 3)
        self.assertTrue(np.allclose(test, self.ref, atol=1e-4,
                                    rtol=1e-4))




# -*- coding: utf-8 -*-
# _ds_hann.py
# This module provides the ds_hann function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the ds_hann() function, used to generate a Hann 
window that does not smear tones located exactly in a bin.
"""

import numpy as np


def ds_hann(n):
    """A Hann window of length :math:`n`. 

    The Hann window, aka *the raised cosine window*, is defined as:

    .. math::

        w(x) = 0.5\\ \\left(1 - cos\\left(\\frac{2 \\pi x}{n}\\right) \\right)

    This windowing function does not smear tones located exactly in a bin.

    **Parameters:**

    n : integer
        The window length, in number of samples.

    **Returns:**

    w : 1d nd_array
        The Hann window.

    .. note:: Functionally equivalent to numpy's ``hanning()``, provided
        to ease porting of code from MATLAB. Also, we take care always to
        return an array of dimensions ``(n,)`` and type ``float_``.

    .. plot::

      import pylab as plt
      from deltasigma import ds_hann
      x = ds_hann(100)
      plt.figure(figsize=(12, 5))
      plt.plot(x, 'o-')
      ax = plt.gca()
      ax.set_ylim(0, 1.02)
      plt.grid(True)
      plt.title("100-samples Hann window")
      plt.xlabel("Sample #")
      plt.ylabel("Value")

    """
    x = np.arange(n, dtype='float_')
    return .5*(1 - np.cos(2*np.pi*x/n))



# -*- coding: utf-8 -*-
# test_evalTFP.py
# This module provides the tests for the evalTFP function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the evalTFP() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class TestEvalTFP(unittest.TestCase):
    """Test class for evalTFP()"""
    def setUp(self):
        pass

    def test_evalTFP_1(self):
        """Test function for evalTFP() 1/2"""
        #          (z-0.3)
        # H1 = ---------------
        #      (z-0.5) (z-0.9)
        H1 = (np.array([.3]), np.array([.5, .9]), 1)
        #             1
        # H2 = ---------------
        #      (z-0.3) (z-0.9)
        H2 = (np.array([]), np.array([.3, .9]), 1)
        a = ds.evalTFP(H1, H2, .2)
        at = np.array([0.5611 + 0.1483j])
        self.assertTrue(np.allclose(np.array([a]), at, atol=1e-4, rtol=1e-4))
        self.assertTrue(np.isscalar(a))

    def test_evalTFP_2(self):
        """Test function for evalTFP() 2/2"""
        #          (z-0.301)
        # H1 = ---------------
        #      (z-0.5) (z-0.9)
        H1 = (np.array([.301]), np.array([.5, .9]), 1)
        #             1
        # H2 = ---------------
        #      (z-0.3) (z-0.9)
        H2 = (np.array([]), np.array([.3, .9]), 1)
        a = ds.evalTFP(H1, H2, np.array([.2, .23, .4]))
        at = np.array([0.5610 + 0.1488j, 0.4466 + 0.0218j, 0.0632 - 0.1504j])
        self.assertTrue(np.allclose(a, at, atol=1e-4, rtol=1e-4))
        self.assertTrue((3,) == a.shape)




# -*- coding: utf-8 -*-
# test_padt.py
# This module provides the tests for the padt function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the padt() function.
"""

import unittest
import numpy as np

from deltasigma import padt

class TestPadt(unittest.TestCase):
    """Test class for padt()"""
    def setUp(self):
        self.tv1 = np.eye(15)
        self.tv2 = np.arange(10)
        self.tv3 = np.array([])

    def test_padt_1(self):
        """Test function for padt() 1/3"""
        tr = padt(self.tv1, n=25, val=2)
        res = np.concatenate((2.*np.ones((10, 15)), self.tv1), axis=0)
        self.assertTrue(np.allclose(tr, res, atol=1e-8, rtol=1e-5))

    def test_padt_2(self):
        """Test function for padt() 2/3"""
        # 1-d array
        tr = padt(self.tv2, n=25, val=1.5)
        res = np.vstack((1.5*np.ones((15, 1)), self.tv2.reshape((-1, 1))))
        self.assertTrue(np.allclose(tr, res, atol=1e-8, rtol=1e-5))

    def test_padt_3(self):
        """Test function for padt() 3/3"""
        # empty matrix array
        tr = padt(self.tv3, n=25, val=1.5)
        res = np.vstack((1.5*np.ones((25, 1)), self.tv3.reshape((-1, 1))))
        self.assertTrue(np.allclose(tr, res, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# test_thermometer.py
# This module provides the tests for the thermometer function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the thermometer() function.
"""

import unittest
import numpy as np

from deltasigma import thermometer

class TestThermometer(unittest.TestCase):
    """Test class for thermometer()"""

    def setUp(self):
        self.tv = np.arange(50)
        self.rm = np.zeros((70, 50))
        for i in range(50):
            self.rm[:i, i] = np.ones(self.rm[:i, i].shape)

    def test_thermometer(self):
        """Test function for thermometer() 1/3"""
        self.assertTrue(np.allclose(thermometer(self.tv, 70), self.rm,
                                    rtol=1e-05, atol=1e-08))




# -*- coding: utf-8 -*-
# test_evalRPoly.py
# This module provides the tests for the evalRPoly function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds

class TestEvalRPoly(unittest.TestCase):
    """Test function for evalRPoly()"""
    def setUp(self):
        pass

    def test_evalRPoly(self):
        """test_evalRPoly"""
        x = np.arange(1001) - 500
        a = [1, 0, 1, 2]
        r1 = np.polyval(a, x)
        rts = np.roots(a)
        r2 = ds.evalRPoly(rts, x)
        self.assertTrue(np.allclose(r1, r2, atol=1e-8, rtol=1e-5))



# -*- coding: utf-8 -*-
# test_evalTF.py
# This module provides the tests for the evalTF function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds

from scipy.signal import lti, tf2zpk
from deltasigma._utils import empty

from nose.tools import raises

class TestEvalTF(unittest.TestCase):
    """Test functions for evalTF()"""
    def setUp(self):
        num = np.poly([3, 0.3, 1])
        den = np.poly([2, 0.5, .25])
        H = (num, den)
        tstr1 = empty()
        (tstr1.form, tstr1.num, tstr1.den) = ('coeff', num, den)
        tstr2 = empty()
        tstr2.form = 'zp'
        (tstr2.zeros, tstr2.poles, tstr2.gain) = tf2zpk(num, den)
        z = np.exp(1j * np.linspace(0, 2*np.pi, num=129, endpoint=True))
        self.h1 = ds.evalTF(tstr1, z)
        self.h2 = ds.evalTF(tstr2, z)
        self.h3 = ds.evalTF(H, z)
        self.h4 = ds.evalTF(lti(tstr2.zeros, tstr2.poles, tstr2.gain), z)
        h5tf = lti(tstr2.zeros, tstr2.poles, tstr2.gain)
        self.h5 = ds.evalTF((h5tf.A, h5tf.B, h5tf.C, h5tf.D), z)
        h6tf = np.vstack((np.hstack((h5tf.A, h5tf.B)),
                          np.hstack((h5tf.C, np.atleast_2d(h5tf.D)))))
        self.h6 = ds.evalTF(h6tf, z)

    def test_evalTF_first(self):
        """Test evalTF num/den obj form"""
        self.assertTrue(np.allclose(
            np.abs(self.h1), np.abs(self.h2), atol=1e-8, rtol=1e-5))

    def test_evalTF_second(self):
        """Test evalTF zp obj form"""
        self.assertTrue(np.allclose(self.h1, self.h3, atol=1e-8, rtol=1e-5))

    def test_evalTF_third(self):
        """Test evalTF zpk and num/den tuple form"""
        self.assertTrue(np.allclose(self.h3, self.h4, atol=1e-8, rtol=1e-5))

    def test_evalTF_fourth(self):
        """Test evalTF ABCD tuple form"""
        self.assertTrue(np.allclose(self.h5, self.h4, atol=1e-8, rtol=1e-5))

    def test_evalTF_fifth(self):
        """Test evalTF ABCD matrix form"""
        self.assertTrue(np.allclose(self.h6, self.h5, atol=1e-8, rtol=1e-5))

    @raises(TypeError)
    def test_evalTF_sixth(self):
        """Test unknown forms error"""
        ds.evalTF(empty(), [1, 2, 3])



# -*- coding: utf-8 -*-
# test_dbp.py
# This module provides the tests for the dbp function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the dbp() function.
"""

import unittest
import numpy as np

from deltasigma import dbp

class TestDbp(unittest.TestCase):
    """Test class for dbp()"""
    def setUp(self):
        self.tv1 = np.array([2])
        self.r1 = np.array([3.01029996])
        self.tv2 = 2
        self.r2 = 3.01029996
        self.tv3 = 2, 2
        self.r3 = 3.01029996, 3.01029996

    def test_dbp_1(self):
        """Test function for dbp() 1/3"""
        res = dbp(self.tv1)
        self.assertTrue(np.allclose(self.r1, res, atol=1e-8, rtol=1e-5))

    def test_dbp_2(self):
        """Test function for dbp() 2/3"""
        res = dbp(self.tv2)
        self.assertTrue(np.allclose(self.r2, res, atol=1e-8, rtol=1e-5))
        self.assertTrue(np.isscalar(res))  # check for type coherence

    def test_dbp_3(self):
        """Test function for dbp() 3/3"""
        res = dbp(self.tv3)
        self.assertTrue(np.allclose(self.r3, res, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# test_DocumentNTF.py
# This module provides the tests for the DocumentNTF() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds
from deltasigma._synthesizeNTF import synthesizeNTF
from deltasigma._realizeNTF import realizeNTF
from deltasigma._stuffABCD import stuffABCD


class TestDocumentNTF(unittest.TestCase):
    """Test function for DocumentNTF()"""

    def setUp(self):
        order = 4
        self.osr = 64
        f0 = 0.0
        Hinf = 1.5
        form = 'CRFB'
        ntf1 = synthesizeNTF(order, self.osr, 2, Hinf, f0)
        self.ntf2 = synthesizeNTF(order, self.osr, 2, Hinf, f0=.333)
        a, g, b, c = realizeNTF(ntf1, form)
        # Use a single feed-in for the input
        b = np.concatenate((
            np.atleast_1d(b[0]), np.zeros((max(b.shape) - 1))
            ))
        self.ABCD = stuffABCD(a, g, b, c, form)

    def test_documentNTF1(self):
        """Test function for DocumentNTF(): check plot with f0 = 0 1/2"""
        # check that DocumentNTF plots with no errors.
        f0 = 0.0
        self.assertIsNone(ds.DocumentNTF(self.ABCD, self.osr, f0))

    def test_documentNTF2(self):
        """Test function for DocumentNTF(): check plot with f0 = 0.333 2/2"""
        f0 = 0.333
        self.assertIsNone(ds.DocumentNTF(self.ntf2, self.osr, f0))




# -*- coding: utf-8 -*-
# test_simulateSNR.py
# This module provides the tests for the simulateSNR function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the simulateSNR() function.
"""

import unittest
import pkg_resources
import scipy.io
import numpy as np
import deltasigma as ds

from scipy.signal import lti

class TestSimulateSNR(unittest.TestCase):
    """Test class for simulateSNR()"""

    def setUp(self):
        pass

    def test_simulateSNR_1(self):
        """Test function for simulateSNR() 1/3"""
        # first test: f0 = 0
        # Load test references
        fname = pkg_resources.resource_filename(__name__,
                                                "test_data/test_snr_amp.mat")
        amp_ref = scipy.io.loadmat(fname)['amp'].reshape((-1,))
        snr_ref = scipy.io.loadmat(fname)['snr'].reshape((-1,))
        amp_user_ref = scipy.io.loadmat(fname)['amp_user'].reshape((-1,))
        snr_user_ref = scipy.io.loadmat(fname)['snr_user'].reshape((-1,))

        order = 4
        osr = 256
        nlev = 2
        f0 = 0.22
        Hinf = 1.25
        form = 'CRFB'

        ntf = ds.synthesizeNTF(order, osr, 2, Hinf, f0)
        a1, g1, b1, c1 = ds.realizeNTF(ntf, form)
        ABCD = ds.stuffABCD(a1, g1, b1, c1, form)

        ABCD_ref = np.array([[1., -1.6252, 0, 0, -0.0789, 0.0789],
                             [1., -0.6252, 0, 0, -0.0756, 0.0756],
                             [0, 1., 1., -1.6252, -0.2758, 0.2758],
                             [0, 1., 1., -0.6252, 0.0843, -0.0843],
                             [0, 0, 0, 1., 1., 0]])
        self.assertTrue(np.allclose(ABCD, ABCD_ref, atol=9e-5, rtol=1e-4))

        # bonus test, mapABCD - realizeNTF - stuffABCD
        a2, g2, b2, c2 = ds.mapABCD(ABCD, form)
        self.assertTrue(np.allclose(a1, a2, atol=1e-5, rtol=1e-5))
        self.assertTrue(np.allclose(g1, g2, atol=1e-5, rtol=1e-5))
        self.assertTrue(np.allclose(b1, b2, atol=1e-5, rtol=1e-5))
        self.assertTrue(np.allclose(c1, c2, atol=1e-5, rtol=1e-5))

        # We do three tests:
        # SNR from ABCD matrix
        # SNR from NTF
        # SNR from LTI obj with user specified amplitudes
        snr, amp = ds.simulateSNR(ABCD, osr, None, f0, nlev)
        self.assertTrue(np.allclose(snr, snr_ref, atol=1, rtol=5e-2))
        self.assertTrue(np.allclose(amp, amp_ref, atol=5e-1, rtol=1e-2))
        snr2, amp2 = ds.simulateSNR(ntf, osr, None, f0, nlev)
        self.assertTrue(np.allclose(snr2, snr_ref, atol=1e-5, rtol=1e-5))
        self.assertTrue(np.allclose(amp2, amp_ref, atol=1e-5, rtol=1e-5))
        amp_user = np.linspace(-100, 0, 200)[::10]
        snr_user, amp_user = ds.simulateSNR(lti(*ntf), osr=osr, amp=amp_user,
                                            f0=f0, nlev=nlev)
        self.assertTrue(np.allclose(snr_user, snr_user_ref[::10], atol=1e-5,
                                    rtol=1e-5))
        self.assertTrue(np.allclose(amp_user, amp_user_ref[::10], atol=1e-5,
                                    rtol=1e-5))

    def test_simulateSNR_2(self):
        """Test function for simulateSNR() 2/3"""
        # next test: f0 = 0
        # Load test references
        fname = pkg_resources.resource_filename(__name__,
                                                "test_data/test_snr_amp2.mat")
        amp_ref = scipy.io.loadmat(fname)['amp'].reshape((-1,))
        snr_ref = scipy.io.loadmat(fname)['snr'].reshape((-1,))
        ABCD_ref = scipy.io.loadmat(fname)['ABCD'].reshape((4, 5))

        order = 3
        osr = 256
        nlev = 2
        f0 = 0.
        Hinf = 1.25
        form = 'CIFB'

        ntf = ds.synthesizeNTF(order, osr, 2, Hinf, f0)

        a1, g1, b1, c1 = ds.realizeNTF(ntf, form)
        a1_ref = [0.008863535715733, 0.093216950269955, 0.444473912607388]
        g1_ref = [9.035620546615189e-05]
        b1_ref = [0.008863535715733, 0.093216950269955, 0.444473912607388, 1.]
        c1_ref = [1., 1., 1.]
        self.assertTrue(np.allclose(a1, a1_ref, atol=1e-9, rtol=5e-5))
        self.assertTrue(np.allclose(g1, g1_ref, atol=1e-9, rtol=5e-5))
        self.assertTrue(np.allclose(b1, b1_ref, atol=1e-9, rtol=1e-4))
        self.assertTrue(np.allclose(c1, c1_ref, atol=1e-9, rtol=2e-5))

        ABCD = ds.stuffABCD(a1, g1, b1, c1, form)
        self.assertTrue(np.allclose(ABCD, ABCD_ref, atol=9e-5, rtol=1e-4))
        snr, amp = ds.simulateSNR(ABCD, osr, None, f0, nlev)
        self.assertTrue(np.allclose(snr, snr_ref, atol=1e-5, rtol=1e-5))
        self.assertTrue(np.allclose(amp, amp_ref, atol=1e-5, rtol=1e-5))

    def test_simulateSNR_3(self):
        """Test function for simulateSNR() 3/3"""
        # next test: amp is a scalar
        fname = pkg_resources.resource_filename(__name__,
                                                "test_data/test_snr_amp2.mat")
        amp_ref = scipy.io.loadmat(fname)['amp'].reshape((-1,))[0]
        snr_ref = scipy.io.loadmat(fname)['snr'].reshape((-1,))[0]
        ABCD = scipy.io.loadmat(fname)['ABCD'].reshape((4, 5))

        order = 3
        osr = 256
        nlev = 2
        f0 = 0.
        Hinf = 1.25
        form = 'CIFB'
        ntf = ds.synthesizeNTF(order, osr, 2, Hinf, f0)

        snr, amp = ds.simulateSNR(ABCD, osr, amp_ref, f0, nlev)
        self.assertTrue(np.allclose(snr, snr_ref, atol=1e-5, rtol=1e-5))
        self.assertTrue(np.allclose(amp, amp_ref, atol=1e-5, rtol=1e-5))




# -*- coding: utf-8 -*-
# _rmsGain.py
# Module providing the rmsGain function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the rmsGain() function
"""

import numpy as np
from scipy.linalg import norm

from ._evalTF import evalTF


def rmsGain(H, f1, f2, N=100):
    """Compute the root mean-square gain of a discrete-time TF.

    The computation is carried out over the frequency band ``(f1, f2)``,
    employing ``N`` discretization steps.

    **Parameters:**

    H : object
        The discrete-time transfer function. See  :func:`evalTF` for the supported types.

    f1 : scalar
        The start value in Hertz of the frequency band over which the gain is evaluated.

    f2 : scalar
        The end value (inclusive) in Hertz of the aforementioned frequency band.

    N : integer, optional
        The number of discretization points to be taken over specified interval.

    **Returns:**

    Grms : scalar
        The root mean-square gain
    """

    w = np.linspace(2*np.pi*f1, 2*np.pi*f2, N)
    g = norm(evalTF(H, np.exp(1j*w))) / np.sqrt(N)

    return g



# -*- coding: utf-8 -*-
# _frespF1.py
# Module providing the frespF1 function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the frespF1() function
"""

from __future__ import division

import numpy as np
import pylab as plt

from ._dbv import dbv


def frespF1(f1, f=None, phi=1, plot=False):
    """Plot/calculate the frequency response of the F1 filter 
    in a Saramaki HBF at the points given in the optional f (n by 1) vector.
    """
    if f is None:
        f = np.linspace(0, 0.5)
    cos_w = np.cos(2*np.pi*f)
    F1 = 0.5
    for i in range(max(f1.shape)):
        F1 = F1 + f1[i] * ((cos_w/phi)**(2*i + 1))
    if plot:
        plt.plot(f, dbv(F1))
        plt.grid('on')
    fresp = F1
    return fresp



# -*- coding: utf-8 -*-
# _synthesizeNTF1.py
# Module providing the synthesizeNTF1 function
# This file is distributed with python-deltasigma.
# Copyright 2013 Giuseppe Venturini
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.
#
# The following code has been taken with little modifications from pydsm,
# its original copyright notice follows:
#
# Copyright (c) 2012, Sergio Callegari
# All rights reserved.
#
# The code was ported from the MATLAB Delta Sigma toolbox, which is
# Copyright (c) 2009, Richard Schreier
#
# The three software follow the same license, known as the 2-clause BSD.
# See the LICENSE file for details.

"""
Module providing the synthesizeNTF1() function.
"""

# The following code is
# Copyright (c) 2012, Sergio Callegari
# All rights reserved.

# Portions of code ported from the DELSIG toolbox
# Copyright (c) 2009, Richard Schreier

from warnings import warn

import numpy as np
from scipy.optimize import fmin_l_bfgs_b

from ._constants import eps
from ._ds_optzeros import ds_optzeros
from ._ds_synNTFobj1 import ds_synNTFobj1
from ._evalTF import evalTF
from ._padl import padl
from ._utils import cplxpair


def synthesizeNTF1(order, osr, opt, H_inf, f0):
    """
    Synthesize a noise transfer function (NTF) for a delta-sigma modulator optimizing the result.
    """
    # Determine the zeros.
    if f0 != 0:
        # Bandpass design-- halve the order temporarily.
        order = order/2
        dw = np.pi/(2*osr)
    else:
        dw = np.pi/osr

    if opt.ndim == 0:
        # opt is a number
        if opt == 0:
            z = np.zeros(order)
        else:
            z = dw*ds_optzeros(order, 1 + np.fmod(opt-1, 2))
        if z.size == 0:
            raise ValueError('Cannot synthesize NTF zeros')
        if f0 != 0:
            # Bandpass design-- shift and replicate the zeros.
            order = order*2
            z = np.sort(z) + 2*np.pi*f0
            z = np.vstack((z,-z)).transpose().flatten()
        z = np.exp(1j*z)
    else:
        z = opt

    zp = z[np.angle(z) > 0]
    x0 = (np.angle(zp)-2*np.pi*f0) * osr / np.pi
    if opt.size == 1 and opt == 4 and f0 != 0:
        # Do not optimize the zeros at f0
        x0 = np.delete(x0, np.nonzero(abs(x0) < eps))

    p = np.zeros(order)
    k = 1
    Hinf_itn_limit = 100
    fprev = 0

    opt_iteration = 5   # Max number of zero-optimizing/Hinf iterations
    while opt_iteration > 0:
        # Iteratively determine the poles by finding the value of the x
        # parameter which results in the desired H_inf
        ftol = 1e-10
        if f0 > 0.25:
            z_inf = 1
        else:
            z_inf = -1
        if f0 == 0:
            # Lowpass design
            HinfLimit = 2**order
            # !!! The limit is actually lower for opt=1 and low OSR
            if H_inf >= HinfLimit:
                warn('Unable to achieve specified Hinf.\n'
                    'Setting all NTF poles to zero.')
                p = np.zeros(order)
            else:
                x = 0.3**(order-1)   # starting guess
                for itn in range(1, Hinf_itn_limit+1):
                    me2 = -0.5*(x**(2./order))
                    w = (2*np.arange(1,order+1)+1)*np.pi/order
                    mb2 = 1+me2*np.exp(1j*w)
                    p = mb2 - np.sqrt(mb2**2-1)
                    # Reflect poles to be inside the unit circle
                    out = abs(p)>1
                    p[out] = 1/p[out]
                    # The following is not exactly what delsig does.
                    # We do not have an identical cplxpair
                    p = cplxpair(p)
                    f = np.real(evalTF((z, p, k), z_inf))-H_inf
                    if itn == 1:
                        delta_x = -f/100
                    else:
                        delta_x = -f*delta_x/(f-fprev)

                    xplus = x+delta_x
                    if xplus > 0:
                        x = xplus
                    else:
                        x = x*0.1
                    fprev = f
                    if abs(f) < 1e-10 or abs(delta_x) < 1e-10:
                        break
                    if x > 1e6:
                        warn('Unable to achieve specified Hinf.\n'
                             'Setting all NTF poles to zero.')
                        p = np.zeros(order)
                        break
                    if itn == Hinf_itn_limit:
                        warn('Danger! Iteration limit exceeded.')
        else:
            # Bandpass design
            x = 0.3**(order/2-1)   # starting guess (not very good for f0~0)
            if f0 > 0.25:
                z_inf = 1.
            else:
                z_inf = -1.
            c2pif0 = np.cos(2*np.pi*f0)
            for itn in range(1, Hinf_itn_limit+1):
                e2 = 0.5*x**(2./order)
                w = (2*np.arange(order)+1)*np.pi/order
                mb2 = c2pif0 + e2*np.exp(1j*w)
                p = mb2 - np.sqrt(mb2**2-1)
                # Reflect poles to be inside the unit circle
                out = abs(p)>1
                p[out] = 1/p[out]
                # The following is not exactly what delsig does.
                p = cplxpair(p)
                f = np.real(evalTF((z, p, k), z_inf))-H_inf
                if itn == 1:
                    delta_x = -f/100
                else:
                    delta_x = -f*delta_x/(f-fprev)
                xplus = x+delta_x
                if xplus > 0:
                    x = xplus
                else:
                    x = x*0.1
                fprev = f
                if abs(f) < 1e-10 or abs(delta_x) < 1e-10:
                    break
                if x > 1e6:
                    warn('Unable to achieve specified Hinf.\n'
                        'Setting all NTF poles to zero.')
                    p = np.zeros(order)
                    break
                if itn == Hinf_itn_limit:
                    warn('Danger! Iteration limit exceeded.')

        # ---- Zero optimization part
        if (opt.size == 1 and opt < 3) or opt.size > 1:
            # Do not optimize the zeros
            opt_iteration = 0
        else:
            if f0 == 0:
                ub = np.ones(x0.size)
                lb = np.zeros(x0.size)
            else:
                ub = 0.5*np.ones(x0.size)
                lb = -ub
            # options = optimset('TolX',0.001, 'TolFun',0.01, 'MaxIter',100 );
            # options = optimset(options,'LargeScale','off');
            # options = optimset(options,'Display','off');
            # %options = optimset(options,'Display','iter');
            opt_result = fmin_l_bfgs_b(ds_synNTFobj1, x0, args=(p, osr, f0),
                                      approx_grad=True, bounds=list(zip(lb,ub)))
            x=opt_result[0]
            x0 = x
            z = np.exp(2j*np.pi*(f0+0.5/osr*x))
            if f0 > 0:
                z = padl(z, len(p)/2, np.exp(2j*np.pi*f0))
            z = np.concatenate((z, z.conj()), axis=1)
            if f0 == 0:
                z = padl(z, len(p), 1)
            if  np.abs(np.real(evalTF((z, p, k), z_inf)) - H_inf ) < ftol:
                opt_iteration = 0
            else:
                opt_iteration = opt_iteration - 1
    z = cplxpair(z)
    return (z, p, k)



#!/usr/bin/env python

import os
from setuptools import setup, find_packages
__version__ = "0.1-8"

def read(fname):
    try:
        with open(os.path.join(os.path.dirname(__file__), fname)) as fp:
            return fp.read()
    except IOError:
        return ""

setup(
    name='deltasigma',
    version=__version__,
    packages=find_packages(),
    package_data={
      'deltasigma': ['tests/test_data/*.mat', 'tests/test_data/*.txt']
    },
    install_requires=['numpy', 'scipy>=0.11.0', 'matplotlib>=1.1.1'],
    zip_safe=False,
    include_package_data=True,
    author="Giuseppe Venturini and others",
    author_email="giuseppe.g.venturini@ieee.org",
    description="a Python package to synthesize, simulate, scale and map " + \
                "to implementable topologies delta sigma modulators.",
    long_description=''.join([read('pypi_description.rst'), '\n\n',
                              read('CHANGES.rst')]),
    license="BSD",
    keywords="delta sigma modulator simulator",
    url="http://github.com/ggventurini/python-deltasigma",
    test_suite = "deltasigma.tests",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4"]
)




# -*- coding: utf-8 -*-
# test_scaleABCD.py
# This module provides the tests for the scaleABCD function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the scaleABCD() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class TestScaleABCD(unittest.TestCase):
    """Test class for scaleABCD()"""

    def setUp(self):
        order = 8
        osr = 32
        self.nlev = 2
        self.f0 = 0.125
        Hinf = 1.5
        form = 'CRFB'
        # Optimized zero placement
        ntf = ds.synthesizeNTF(order, osr, 2, Hinf, self.f0)
        a, g, b, c = ds.realizeNTF(ntf, form)
        # we pass b = b[0]
        # if you use a single feed-in for the input, b may be scalar too
        ABCD = ds.stuffABCD(a, g, b[0], c, form)
        # now we correct b for the assert
        b = np.concatenate(( # Use a single feed-in for the input
                            np.atleast_1d(b[0]),
                            np.zeros((max(b.shape) - 1,))
                          ))
        self.ABCD0 = ABCD.copy()

        # References
        self.Sdiag_ref = np.array([71.9580, 51.9359, 8.2133, 6.5398,
                                   1.9446, 1.2070, 0.4223, 0.3040])
        self.umax_ref = 0.8667
        ABCD_ref1 = np.array([[1., -0.7320, 0, 0, 0, 0],
                              [0.7218, 0.4717, 0, 0, 0, 0],
                              [0, 0.1581, 1., -0.7357, 0, 0],
                              [0, 0.1259, 0.7962, 0.4142, 0, 0],
                              [0, 0, 0, 0.2973, 1., -0.9437],
                              [0, 0, 0, 0.1846, 0.6207, 0.4142],
                              [0, 0, 0, 0, 0, 0.3499],
                              [0, 0, 0, 0, 0, 0.2518],
                              [0, 0, 0, 0, 0, 0]])
        ABCD_ref2 = np.array([[0, 0, 0.0858, -0.0858],
                              [0, 0, 0.0619, 0.0428],
                              [0, 0, 0, 0.0642],
                              [0, 0, 0, 0.1835],
                              [0, 0, 0, 0.2447],
                              [0, 0, 0, 0.0581],
                              [1., -0.8971, 0, -0.0076],
                              [0.7197, 0.3543, 0, -0.1746],
                              [0, 3.29, 0, 0]])
        self.ABCD_ref = np.hstack((ABCD_ref1, ABCD_ref2))

    def test_scaleABCD(self):
        """Test function for scaleABCD()"""
        ABCD, umax, S = ds.scaleABCD(self.ABCD0, self.nlev, self.f0)
        # mapping the NTF to states, there is not a perfect match between 
        # the original code and the scipy version. -> rtol approx 20%
        #if not np.allclose(ABCD, ABCD_ref, atol=1e-2, rtol=3e-1):
        #    aerr = ABCD_ref-ABCD
        #    rerr = 2*(ABCD_ref-ABCD)/(ABCD_ref+ABCD)
        #    print(repr(ABCD_ref))
        #    print(repr(ABCD))
        #    print(aerr)
        #    print(rerr)
        # this is a rather high relative error. We get it on Travis-CI
        # Probably related to the libs used?
        self.assertTrue(np.allclose(ABCD, self.ABCD_ref, atol=1e-2, rtol=30e-2))
        self.assertTrue(np.allclose(umax, self.umax_ref, atol=1e-4, rtol=1e-3))
        self.assertTrue(np.allclose(np.diag(S), self.Sdiag_ref, atol=1e-2, rtol=25e-1))

        




# -*- coding: utf-8 -*-
# _circ_smooth.py
# Module providing the circ_smooth function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the circ_smooth() function
"""

from __future__ import division

import numpy as np

from ._ds_hann import ds_hann
from ._utils import circshift


def circ_smooth(x, n=16):
    """Smoothing of the PSD ``x`` for linear x-axis plotting.

    **Parameters:**

    x : 1D ndarray
        The PSD to be smoothed, or equivalently

    .. math::

        x = \\left|\\mathrm{FFT}\\left(u(t)\\right)(f)\\right|^2

    n : int, even, optional
        The length of the Hann window used in the smoothing algorithm.

    **Returns:**

    y : 1D ndarray
        The smoothed PSD

    .. seealso::

        :func:`logsmooth`, smoothing algorithm suitable for logarithmic x-axis
        plotting.

    For a comparison of :func:`circ_smooth` and :func:`logsmooth` (accessed
    through the helper function :func:`plotSpectrum`) see the following plot.

    .. plot::

        import numpy as np
        from deltasigma import *
        from numpy.fft import fft
        f0 = 0
        osr = 32
        quadrature = False
        Hinf = 1.5
        order = 3
        ntf = synthesizeNTF(order, osr, 0, Hinf, f0)
        f1, f2 = ds_f1f2(osr, f0, quadrature)
        delta = 2
        Amp = undbv(-3)
        f = 0.3
        N = 2**12
        f1_bin = np.round(f1*N)
        f2_bin = np.round(f2*N)
        fin = np.round(((1 - f)/2*f1 + (f + 1)/2*f2) * N)
        t = np.arange(0, N)
        u = Amp*np.cos((2*np.pi/N)*fin*t)
        v, xn, xmax, y = simulateDSM(u, ntf, 2)
        window = ds_hann(N)
        NBW = 1.5/N
        spec0 = fft(v * window)/(N/4)
        freq = np.linspace(0, 0.5, N/2 + 1)
        # plotting
        plt.figure(figsize=(12, 7))
        plt.subplot(211)
        plt.plot(freq, dbv(spec0[:N/2 + 1]), 'c', linewidth=1, label='$S$')
        plt.hold(True)
        spec_smoothed = circ_smooth(np.abs(spec0)**2., 16)
        plt.plot(freq, dbp(spec_smoothed[:N/2 + 1]), 'b--', linewidth=2, label='$\\\\mathrm{circ\\\\_smooth}(S)$')
        plotSpectrum(spec0, fin, 'r', linewidth=2, label='$\\\\mathrm{plotSpectrum}(S)$')
        Snn = np.abs(evalTF(ntf, np.exp(2j*np.pi*freq)))**2 * 2/12*(delta)**2
        plt.plot(freq, dbp(Snn*NBW), 'm', linewidth=1.5, label='$\\\\mathrm{from\\\\ NTF}$')
        snr = calculateSNR(spec0[f1_bin:f2_bin + 1], fin - f1_bin)
        msg = 'SQNR  =  %.1fdB\\n @ A = %.1fdBFS & osr = %.0f\\n' % \\
              (snr, dbv(spec0[fin]), osr)
        plt.text(f0 + .45/osr, -20, msg, horizontalalignment='left',
                 verticalalignment='center')
        plt.text(0.5, -3, 'NBW = %.1e ' % NBW, horizontalalignment='right',
                 verticalalignment='top')
        figureMagic((0, 0.5), None, None, (-140, 0), 20, None)
        plt.ylabel('Spectrum [dB]')
        ax = plt.gca()
        ax.set_title('Smoothing and plotting for LOG and LIN axes')
        plt.legend(loc=4)
        plt.subplot(212)
        plt.plot(freq, dbv(spec0[:N/2 + 1]), 'c', linewidth=1, label='$S$')
        plt.hold(True)
        plotSpectrum(spec0, fin, '--r', linewidth=2, label='$\\\\mathrm{plotSpectrum}(S)$')
        plt.plot(freq, dbp(spec_smoothed[:N/2 + 1]), 'b', linewidth=2, label='$\\\\mathrm{circ\\\\_smooth}(S)$')
        plt.plot(freq, dbp(Snn*NBW), 'm', linewidth=1.5, label='$\\\\mathrm{from\\\\ NTF}$')
        msg = 'SQNR  =  %.1fdB\\n @ A = %.1fdBFS & osr = %.0f\\n' % \\
              (snr, dbv(spec0[fin]), osr)
        plt.text(f0 + 1./osr, -20, msg, horizontalalignment='left',
                 verticalalignment='center')
        plt.text(0.5, -3, 'NBW = %.1e ' % NBW, horizontalalignment='right',
                 verticalalignment='top')
        figureMagic((0, 0.5), None, None, (-140, 0), 20, None)
        ax = plt.gca()
        ax.set_xscale('linear')
        plt.ylabel('Spectrum [dB]')
        plt.xlabel('Normalized frequency ($f_s \\\\rightarrow 1$)')
        plt.legend(loc=4)

    """
    assert len(x.shape) == 1 or 1 in x.shape
    assert n % 2 == 0
    nx = max(x.shape)
    w = ds_hann(n)/(n/2.)
    xw = np.convolve(x, w)
    yp = np.hstack((xw[n - 1:nx], xw[:n - 1] + xw[nx:]))
    y = circshift(yp, [int(n/2. - 1)])
    return y



# -*- coding: utf-8 -*-
# _plotSpectrum.py
# Module providing the plotSpectrum function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the plotSpectrum() function
"""
from __future__ import division

import pylab as plt

from ._logsmooth import logsmooth


def plotSpectrum(X, fin, fmt='-', **xargs):
    """Plot a smoothed spectrum on a LOG x-axis.

    **Parameters:**

    X : 1D ndarray
        The FFT to be smoothed and plotted, *dual sided*.

    fin : int
        The bin corresponding to the input sine wave.

    fmt : string, optional
        Formatting to be passed to matplotlib's ``semilogx()``.

    **xargs : dict, optional
        Extra arguments to be passed to matplotlib's ``semilogx()``.

    Plotting is performed on the current figure.

    .. seealso::

         * :func:`logsmooth` for more information on the smoothing algorithm

         * :func:`circ_smooth` for a smoothing algorithm suitable for linear x-axes.

    .. plot::

        import numpy as np
        from deltasigma import *
        from numpy.fft import fft
        f0 = 0
        osr = 32
        quadrature = False
        Hinf = 1.5
        order = 3
        ntf = synthesizeNTF(order, osr, 0, Hinf, f0)
        f1, f2 = ds_f1f2(osr, f0, quadrature)
        delta = 2
        Amp = undbv(-3)
        f = 0.3
        N = 2**12
        f1_bin = np.round(f1*N)
        f2_bin = np.round(f2*N)
        fin = np.round(((1 - f)/2*f1 + (f + 1)/2*f2) * N)
        t = np.arange(0, N)
        u = Amp*np.cos((2*np.pi/N)*fin*t)
        v, xn, xmax, y = simulateDSM(u, ntf, 2)
        window = ds_hann(N)
        NBW = 1.5/N
        spec0 = fft(v * window)/(N/4)
        freq = np.linspace(0, 0.5, N/2 + 1)
        plt.plot(freq, dbv(spec0[:N/2 + 1]), 'c', linewidth=1, label='$S$')
        plt.hold(True)
        plotSpectrum(spec0, fin, 'b', linewidth=2.5, label='$\\\\mathrm{plotSpectrum}(S)$')
        Snn = np.abs(evalTF(ntf, np.exp(2j*np.pi*freq)))**2 * 2/12*(delta)**2
        plt.plot(freq, dbp(Snn*NBW), 'm', linewidth=1.5, label='$\\\\mathrm{from\\\\ NTF}$')
        snr = calculateSNR(spec0[f1_bin:f2_bin + 1], fin - f1_bin)
        msg = 'SQNR  =  %.1fdB\\n @ A = %.1fdBFS & osr = %.0f\\n' % \\
              (snr, dbv(spec0[fin]), osr)
        plt.text(f0 + .45/osr, -20, msg, horizontalalignment='left',
                 verticalalignment='center')
        plt.text(0.5, -3, 'NBW = %.1e ' % NBW, horizontalalignment='right',
                 verticalalignment='top')
        figureMagic((1e-3, 0.5), None, None, (-140, 0), 20, None)
        plt.ylabel('Spectrum [dB]')
        ax = plt.gca()
        ax.set_title('Third order modulator')
        plt.xlabel('Normalized frequency ($f_s \\\\rightarrow 1$)')
        plt.legend(loc=4)

    """
    f, p = logsmooth(X, fin)
    plt.semilogx(f, p, fmt, **xargs)
